@@ -1,7 +1,13 @@
+-- This config example file is released into the Public Domain.
 
--- This configuration for the flex backend tries to be compatible with the
--- original pgsql c-transform backend. There might be some corner cases but
--- it should mostly do exactly the same.
+-- This configuration for the flex output tries to be compatible with the
+-- original pgsql C transform output. There might be some corner cases but
+-- it should do exactly the same in almost all cases.
+
+-- The output projection used (3857, web mercator is the default). Set this
+-- to 4326 if you were using the -l|--latlong option or to the EPSG
+-- code you were using on the -E|-proj option.
+local srid = 3857
 
 -- Set this to true if you were using option -K|--keep-coastlines.
 local keep_coastlines = false
@@ -9,8 +15,8 @@ local keep_coastlines = false
 -- Set this to the table name prefix (what used to be option -p|--prefix).
 local prefix = 'planet_osm'
 
--- Set this to true if multipolygons should be written as polygons into db
--- (what used to be option -G|--multi-geometry).
+-- Set this to true if multipolygons should be written as multipolygons into
+-- db (what used to be option -G|--multi-geometry).
 local multi_geometry = false
 
 -- Set this to true if you want an hstore column (what used to be option
@@ -24,12 +30,14 @@ local hstore_all = false
 -- Only keep objects that have a value in one of the non-hstore columns
 -- (normal action with --hstore is to keep all objects). Equivalent to
 -- what used to be set through option --hstore-match-only.
-local hstore_match_only = true
+local hstore_match_only = false
 
 -- Set this to add an additional hstore (key/value) column containing all tags
 -- that start with the specified string, eg "name:". Will produce an extra
 -- hstore column that contains all "name:xx" tags. Equivalent to what used to
--- be set through option -z|--hstore-column.
+-- be set through option -z|--hstore-column. Unlike the -z option which can
+-- be specified multiple time, this does only support a single additional
+-- hstore column.
 local hstore_column = nil
 
 -- There is some very old specialized handling of route relations in osm2pgsql,
@@ -44,7 +52,7 @@ if hstore and hstore_all then
 end
 
 -- Used for splitting up long linestrings
-if osm2pgsql.srid == 4326 then
+if srid == 4326 then
     max_length = 1
 else
     max_length = 100000
@@ -342,6 +350,7 @@ function gen_columns(text_columns, with_hstore, area, geometry_type)
     end
 
     add_column('way', geometry_type)
+    columns[#columns].projection = srid
 
     return columns
 end
@@ -718,7 +727,10 @@ function osm2pgsql.process_relation(object)
     end
 
     if make_boundary or make_polygon then
-        output.way = { create = 'area', multi = multi_geometry }
+        output.way = { create = 'area' }
+        if not multi_geometry then
+            output.way.split_at = 'multi'
+        end
         tables.polygon:add_row(output)
     end
 end
