@@ -2,16 +2,7 @@
 
 -- This example shows how you can use JSON and JSONB columns in your database.
 
--- You need the 'dkjson' JSON library for Lua. Install 'dkjson' with LuaRocks
--- or install from http://dkolf.de/src/dkjson-lua.fsl/home . Debian/Ubuntu
--- users can install the 'lua-dkjson' package.
-
--- Use JSON encoder
-local json = require('dkjson')
-
 local places = osm2pgsql.define_node_table('places', {
-    -- The jsonb column is handled by osm2pgsql just like a normal text
-    -- column. It is the job of the Lua script to put correct json there.
     { column = 'tags', type = 'jsonb' },
     { column = 'geom', type = 'point' },
 })
@@ -46,8 +37,13 @@ function osm2pgsql.process_node(object)
         object.tags.names = names
     end
 
+    -- The population should be stored as number, not as a string
+    if object.tags.population then
+        object.tags.population = tonumber(object.tags.population)
+    end
+
     places:add_row({
-        tags = json.encode(object.tags)
+        tags = object.tags
     })
 end
 
