@@ -6,11 +6,11 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
-#include "options.hpp"
+class options_t;
 
 /**
  * Get settings from the database and check that minimum requirements for
