@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -41,7 +41,7 @@ public:
                 m_processor->add_buffer(std::move(m_current));
             }
 
-            m_current.reset(new db_cmd_copy_delete_t<DELETER>{table});
+            m_current = std::make_unique<db_cmd_copy_delete_t<DELETER>>(table);
         }
     }
 
@@ -250,9 +250,10 @@ public:
     {
         char const *const lookup_hex = "0123456789ABCDEF";
 
-        for (char c : wkb) {
-            m_current->buffer += lookup_hex[(c >> 4U) & 0xfU];
-            m_current->buffer += lookup_hex[c & 0xfU];
+        for (auto c : wkb) {
+            auto const num = static_cast<unsigned int>(c);
+            m_current->buffer += lookup_hex[(num >> 4U) & 0xfU];
+            m_current->buffer += lookup_hex[num & 0xfU];
         }
         m_current->buffer += '\t';
     }
