@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -23,11 +23,11 @@ void db_deleter_by_id_t::delete_rows(std::string const &table,
     // Add 50 characters for the SQL statement itself.
     sql.reserve(m_deletables.size() * 15 + 50);
 
-    fmt::format_to(sql, FMT_STRING("DELETE FROM {} WHERE {} IN ("), table,
-                   column);
+    fmt::format_to(std::back_inserter(sql),
+                   FMT_STRING("DELETE FROM {} WHERE {} IN ("), table, column);
 
     for (auto id : m_deletables) {
-        format_to(sql, FMT_STRING("{},"), id);
+        format_to(std::back_inserter(sql), FMT_STRING("{},"), id);
     }
     sql[sql.size() - 1] = ')';
 
@@ -48,11 +48,12 @@ void db_deleter_by_type_and_id_t::delete_rows(std::string const &table,
     sql.reserve(m_deletables.size() * 22 + 200);
 
     if (m_has_type) {
-        fmt::format_to(sql, "DELETE FROM {} p USING (VALUES ", table);
+        fmt::format_to(std::back_inserter(sql),
+                       "DELETE FROM {} p USING (VALUES ", table);
 
         for (auto const &item : m_deletables) {
-            fmt::format_to(sql, FMT_STRING("('{}',{}),"), item.osm_type,
-                           item.osm_id);
+            fmt::format_to(std::back_inserter(sql), FMT_STRING("('{}',{}),"),
+                           item.osm_type, item.osm_id);
         }
 
         sql.resize(sql.size() - 1);
@@ -61,16 +62,17 @@ void db_deleter_by_type_and_id_t::delete_rows(std::string const &table,
         assert(pos != std::string::npos);
         std::string type = column.substr(0, pos);
 
-        fmt::format_to(sql,
+        fmt::format_to(std::back_inserter(sql),
                        ") AS t (osm_type, osm_id) WHERE"
                        " p.{} = t.osm_type AND p.{} = t.osm_id",
                        type, column.c_str() + pos + 1);
     } else {
-        fmt::format_to(sql, FMT_STRING("DELETE FROM {} WHERE {} IN ("), table,
+        fmt::format_to(std::back_inserter(sql),
+                       FMT_STRING("DELETE FROM {} WHERE {} IN ("), table,
                        column);
 
         for (auto const &item : m_deletables) {
-            format_to(sql, FMT_STRING("{},"), item.osm_id);
+            format_to(std::back_inserter(sql), FMT_STRING("{},"), item.osm_id);
         }
         sql[sql.size() - 1] = ')';
     }
@@ -105,15 +107,14 @@ void db_copy_thread_t::sync_and_wait()
 {
     std::promise<void> barrier;
     std::future<void> sync = barrier.get_future();
-    add_buffer(
-        std::unique_ptr<db_cmd_t>(new db_cmd_sync_t{std::move(barrier)}));
+    add_buffer(std::make_unique<db_cmd_sync_t>(std::move(barrier)));
     sync.wait();
 }
 
 void db_copy_thread_t::finish()
 {
     if (m_worker.joinable()) {
-        add_buffer(std::unique_ptr<db_cmd_t>(new db_cmd_finish_t{}));
+        add_buffer(std::make_unique<db_cmd_finish_t>());
         m_worker.join();
     }
 }
@@ -125,11 +126,15 @@ db_copy_thread_t::thread_t::thread_t(std::string conninfo, shared &shared)
 void db_copy_thread_t::thread_t::operator()()
 {
     try {
-        m_conn.reset(new pg_conn_t{m_conninfo});
+        m_conn = std::make_unique<pg_conn_t>(m_conninfo);
 
         // Let commits happen faster by delaying when they actually occur.
         m_conn->exec("SET synchronous_commit = off");
 
+        // Do not show messages about invalid geometries (they are removed
+        // by the trigger).
+        m_conn->exec("SET client_min_messages = WARNING");
+
         bool done = false;
         while (!done) {
             std::unique_ptr<db_cmd_t> item;
@@ -191,9 +196,11 @@ void db_copy_thread_t::thread_t::start_copy(
     fmt::memory_buffer sql;
     sql.reserve(qname.size() + target->rows.size() + 20);
     if (target->rows.empty()) {
-        fmt::format_to(sql, FMT_STRING("COPY {} FROM STDIN"), qname);
+        fmt::format_to(std::back_inserter(sql),
+                       FMT_STRING("COPY {} FROM STDIN"), qname);
     } else {
-        fmt::format_to(sql, FMT_STRING("COPY {} ({}) FROM STDIN"), qname,
+        fmt::format_to(std::back_inserter(sql),
+                       FMT_STRING("COPY {} ({}) FROM STDIN"), qname,
                        target->rows);
     }
 
