@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -116,13 +116,13 @@ void expire_tiles::expire_tile(uint32_t x, uint32_t y)
     }
 }
 
-int expire_tiles::normalise_tile_x_coord(int x)
+uint32_t expire_tiles::normalise_tile_x_coord(int x) const
 {
     x %= map_width;
     if (x < 0) {
         x = (map_width - x) + 1;
     }
-    return x;
+    return static_cast<uint32_t>(x);
 }
 
 void expire_tiles::coords_to_tile(double lon, double lat, double *tilex,
@@ -141,10 +141,10 @@ void expire_tiles::coords_to_tile(double lon, double lat, double *tilex,
 void expire_tiles::from_line(double lon_a, double lat_a, double lon_b,
                              double lat_b)
 {
-    double tile_x_a;
-    double tile_y_a;
-    double tile_x_b;
-    double tile_y_b;
+    double tile_x_a = NAN;
+    double tile_y_a = NAN;
+    double tile_x_b = NAN;
+    double tile_y_b = NAN;
 
     coords_to_tile(lon_a, lat_a, &tile_x_a, &tile_y_a);
     coords_to_tile(lon_b, lat_b, &tile_x_b, &tile_y_b);
@@ -200,10 +200,12 @@ void expire_tiles::from_line(double lon_a, double lat_a, double lon_b,
         }
         for (int x = x1 - TILE_EXPIRY_LEEWAY; x <= x2 + TILE_EXPIRY_LEEWAY;
              ++x) {
-            int const norm_x = normalise_tile_x_coord(x);
+            uint32_t const norm_x = normalise_tile_x_coord(x);
             for (int y = y1 - TILE_EXPIRY_LEEWAY; y <= y2 + TILE_EXPIRY_LEEWAY;
                  ++y) {
-                expire_tile(norm_x, y);
+                if (y >= 0) {
+                    expire_tile(norm_x, static_cast<uint32_t>(y));
+                }
             }
         }
     }
@@ -235,8 +237,8 @@ int expire_tiles::from_bbox(double min_lon, double min_lat, double max_lon,
     }
 
     /* Convert the box's Mercator coordinates into tile coordinates */
-    double tmp_x;
-    double tmp_y;
+    double tmp_x = NAN;
+    double tmp_y = NAN;
     coords_to_tile(min_lon, max_lat, &tmp_x, &tmp_y);
     int min_tile_x = tmp_x - TILE_EXPIRY_LEEWAY;
     int min_tile_y = tmp_y - TILE_EXPIRY_LEEWAY;
@@ -256,22 +258,22 @@ int expire_tiles::from_bbox(double min_lon, double min_lat, double max_lon,
         max_tile_y = map_width;
     }
     for (int iterator_x = min_tile_x; iterator_x <= max_tile_x; ++iterator_x) {
-        int const norm_x = normalise_tile_x_coord(iterator_x);
+        uint32_t const norm_x = normalise_tile_x_coord(iterator_x);
         for (int iterator_y = min_tile_y; iterator_y <= max_tile_y;
              ++iterator_y) {
-            expire_tile(norm_x, iterator_y);
+            expire_tile(norm_x, static_cast<uint32_t>(iterator_y));
         }
     }
     return 0;
 }
 
-void expire_tiles::from_wkb(char const *wkb, osmid_t osm_id)
+void expire_tiles::from_wkb(std::string const &wkb, osmid_t osm_id)
 {
     if (maxzoom == 0) {
         return;
     }
 
-    auto parse = ewkb::parser_t(wkb);
+    ewkb::parser_t parse{wkb};
 
     switch (parse.read_header()) {
     case ewkb::wkb_point:
@@ -377,37 +379,6 @@ void expire_tiles::from_wkb_polygon(ewkb::parser_t *wkb, osmid_t osm_id)
     }
 }
 
-/*
- * Expire tiles based on an osm element.
- * What type of element (node, line, polygon) osm_id refers to depends on
- * sql_conn. Each type of table has its own sql_conn and the prepared statement
- * get_wkb refers to the appropriate table.
- *
- * The function returns -1 if expiry is not enabled. Otherwise it returns the number
- * of elements that refer to the osm_id.
-
- */
-int expire_tiles::from_db(table_t *table, osmid_t osm_id)
-{
-    //bail if we dont care about expiry
-    if (maxzoom == 0) {
-        return -1;
-    }
-
-    //grab the geom for this id
-    auto wkbs = table->get_wkb_reader(osm_id);
-
-    //dirty the stuff
-    char const *wkb = nullptr;
-    while ((wkb = wkbs.get_next())) {
-        auto const binwkb = ewkb::parser_t::wkb_from_hex(wkb);
-        from_wkb(binwkb.c_str(), osm_id);
-    }
-
-    //return how many rows were affected
-    return wkbs.get_count();
-}
-
 int expire_tiles::from_result(pg_result_t const &result, osmid_t osm_id)
 {
     //bail if we dont care about expiry
@@ -420,7 +391,7 @@ int expire_tiles::from_result(pg_result_t const &result, osmid_t osm_id)
     for (int i = 0; i < num_tuples; ++i) {
         char const *const wkb = result.get_value(i, 0);
         auto const binwkb = ewkb::parser_t::wkb_from_hex(wkb);
-        from_wkb(binwkb.c_str(), osm_id);
+        from_wkb(binwkb, osm_id);
     }
 
     //return how many rows were affected
