@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -20,9 +20,10 @@
 class reprojection;
 class table_t;
 class tile;
+
 namespace ewkb {
 class parser_t;
-}
+} // namespace ewkb
 
 /**
  * \brief Simple struct for the x and y index of a tile ID.
@@ -60,10 +61,22 @@ struct expire_tiles
     expire_tiles(uint32_t maxzoom, double maxbbox,
                  const std::shared_ptr<reprojection> &projection);
 
+    bool enabled() const noexcept { return maxzoom != 0; }
+
     int from_bbox(double min_lon, double min_lat, double max_lon,
                   double max_lat);
-    void from_wkb(char const *wkb, osmid_t osm_id);
-    int from_db(table_t *table, osmid_t osm_id);
+    void from_wkb(std::string const &wkb, osmid_t osm_id);
+
+    /**
+     * Expire tiles based on an osm id.
+     *
+     * \param result Result of a database query into some table returning the
+     *               geometries. (This is usally done using the "get_wkb"
+     *               prepared statement.)
+     * \param osm_id The OSM id to look for.
+     * \return The number of elements that refer to the osm_id or -1 if
+     *         expire is disabled.
+     */
     int from_result(pg_result_t const &result, osmid_t osm_id);
 
     /**
@@ -102,11 +115,10 @@ struct expire_tiles
          * (larger than largest possible quadkey). */
         uint64_t last_quadkey = 1ULL << (2 * maxzoom);
         std::size_t count = 0;
-        for (std::vector<uint64_t>::const_iterator it = tiles_maxzoom.cbegin();
-             it != tiles_maxzoom.cend(); ++it) {
+        for (auto const quadkey : tiles_maxzoom) {
             for (uint32_t dz = 0; dz <= maxzoom - minzoom; ++dz) {
                 // scale down to the current zoom level
-                uint64_t qt_current = *it >> (dz * 2);
+                uint64_t qt_current = quadkey >> (dz * 2);
                 /* If dz > 0, there are propably multiple elements whose quadkey
                  * is equal because they are all sub-tiles of the same tile at the current
                  * zoom level. We skip all of them after we have written the first sibling.
@@ -118,7 +130,7 @@ struct expire_tiles
                 output_writer.output_dirty_tile(xy.x, xy.y, maxzoom - dz);
                 ++count;
             }
-            last_quadkey = *it;
+            last_quadkey = quadkey;
         }
         log_info("Wrote {} entries to expired tiles list", count);
     }
@@ -166,7 +178,7 @@ private:
      * \param y y index of the tile to be expired.
      */
     void expire_tile(uint32_t x, uint32_t y);
-    int normalise_tile_x_coord(int x);
+    uint32_t normalise_tile_x_coord(int x) const;
     void from_line(double lon_a, double lat_a, double lon_b, double lat_b);
 
     void from_wkb_point(ewkb::parser_t *wkb);
