@@ -3,16 +3,17 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
 #include "flex-table-column.hpp"
+#include "format.hpp"
 
 #include <algorithm>
 #include <array>
 #include <cctype>
-#include <cstring>
+#include <cstdlib>
 #include <stdexcept>
 #include <utility>
 
@@ -22,9 +23,8 @@ struct column_type_lookup
     table_column_type type;
 };
 
-static std::array<column_type_lookup, 24> const column_types = {
-    {{"sql", table_column_type::sql},
-     {"text", table_column_type::text},
+static std::array<column_type_lookup, 25> const column_types = {
+    {{"text", table_column_type::text},
      {"boolean", table_column_type::boolean},
      {"bool", table_column_type::boolean},
      {"int2", table_column_type::int2},
@@ -36,6 +36,8 @@ static std::array<column_type_lookup, 24> const column_types = {
      {"bigint", table_column_type::int8},
      {"real", table_column_type::real},
      {"hstore", table_column_type::hstore},
+     {"json", table_column_type::json},
+     {"jsonb", table_column_type::jsonb},
      {"direction", table_column_type::direction},
      {"geometry", table_column_type::geometry},
      {"point", table_column_type::point},
@@ -49,7 +51,7 @@ static std::array<column_type_lookup, 24> const column_types = {
      {"id_num", table_column_type::id_num}}};
 
 static table_column_type
-get_column_type_from_string(std::string const &type) noexcept
+get_column_type_from_string(std::string const &type)
 {
     auto const it =
         std::find_if(std::begin(column_types), std::end(column_types),
@@ -57,13 +59,11 @@ get_column_type_from_string(std::string const &type) noexcept
                          return type == name_type.name;
                      });
 
-    if (it != std::end(column_types)) {
-        return it->type;
+    if (it == std::end(column_types)) {
+        throw std::runtime_error{"Unknown column type '{}'."_format(type)};
     }
 
-    // If we don't recognize the column type, we just assume its a valid SQL
-    // type and use it "as is".
-    return table_column_type::sql;
+    return it->type;
 }
 
 static std::string lowercase(std::string const &str)
@@ -79,8 +79,10 @@ static std::string lowercase(std::string const &str)
 }
 
 flex_table_column_t::flex_table_column_t(std::string name,
-                                         std::string const &type)
+                                         std::string const &type,
+                                         std::string const &sql_type)
 : m_name(std::move(name)), m_type_name(lowercase(type)),
+  m_sql_type(sql_type),
   m_type(get_column_type_from_string(m_type_name))
 {}
 
@@ -113,9 +115,11 @@ void flex_table_column_t::set_projection(char const *projection)
 
 std::string flex_table_column_t::sql_type_name() const
 {
+    if (!m_sql_type.empty()) {
+        return m_sql_type;
+    }
+
     switch (m_type) {
-    case table_column_type::sql:
-        return m_type_name;
     case table_column_type::text:
         return "text";
     case table_column_type::boolean:
@@ -130,6 +134,10 @@ std::string flex_table_column_t::sql_type_name() const
         return "real";
     case table_column_type::hstore:
         return "hstore";
+    case table_column_type::json:
+        return "json";
+    case table_column_type::jsonb:
+        return "jsonb";
     case table_column_type::direction:
         return "int2";
     case table_column_type::geometry:
