@@ -6,20 +6,16 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
-#include "format.hpp"
-#include "reprojection.hpp"
-
+#include <cassert>
 #include <cstdint>
 #include <string>
 
 enum class table_column_type : uint8_t
 {
-    sql,
-
     text,
 
     boolean,
@@ -31,6 +27,8 @@ enum class table_column_type : uint8_t
     real,
 
     hstore,
+    json,
+    jsonb,
 
     direction,
 
@@ -54,7 +52,8 @@ enum class table_column_type : uint8_t
 class flex_table_column_t
 {
 public:
-    flex_table_column_t(std::string name, std::string const &type);
+    flex_table_column_t(std::string name, std::string const &type,
+                        std::string const &sql_type);
 
     std::string const &name() const noexcept { return m_name; }
 
@@ -85,6 +84,18 @@ public:
                (m_type <= table_column_type::multipolygon);
     }
 
+    /**
+     * Do we need an ST_IsValid() check in the database for this geometry
+     * column? If the SRID is 4326 the geometry validity is already assured
+     * by libosmium, so we don't need it. And Point geometries are always
+     * valid.
+     */
+    bool needs_isvalid() const noexcept
+    {
+        assert(is_geometry_column());
+        return m_srid != 4326 && m_type != table_column_type::point;
+    }
+
     std::string const &type_name() const noexcept { return m_type_name; }
 
     bool not_null() const noexcept { return m_not_null; }
@@ -108,14 +119,18 @@ private:
     std::string m_name;
 
     /**
-     * The type name of the database table column. Either a name we recognize
-     * or just an SQL snippet.
+     * The type name of the column.
      */
     std::string m_type_name;
 
     /**
-     * The type of database column. Use table_column_type::sql as fallback
-     * in which case m_type_name is the SQL type used in the database.
+     * The SQL type of the database table column. If this is not set, use
+     * one generated from the m_type.
+     */
+    std::string m_sql_type;
+
+    /**
+     * The type of column.
      */
     table_column_type m_type;
 
