@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -14,6 +14,7 @@
 #include "flex-table-column.hpp"
 #include "osmium-builder.hpp"
 #include "pgsql.hpp"
+#include "thread-pool.hpp"
 
 #include <osmium/osm/item_type.hpp>
 
@@ -47,6 +48,11 @@ public:
 
     std::string const &schema() const noexcept { return m_schema; }
 
+    bool cluster_by_geom() const noexcept
+    {
+        return has_geom_column() && m_cluster_by_geom;
+    }
+
     std::string const &data_tablespace() const noexcept
     {
         return m_data_tablespace;
@@ -59,6 +65,11 @@ public:
 
     void set_schema(std::string const &schema) noexcept { m_schema = schema; }
 
+    void set_cluster_by_geom(bool cluster) noexcept
+    {
+        m_cluster_by_geom = cluster;
+    }
+
     void set_data_tablespace(std::string const &tablespace) noexcept
     {
         m_data_tablespace = tablespace;
@@ -172,7 +183,8 @@ public:
     }
 
     flex_table_column_t &add_column(std::string const &name,
-                                    std::string const &type);
+                                    std::string const &type,
+                                    std::string const &sql_type);
 
     bool has_multicolumn_id_index() const noexcept;
     std::string id_column_names() const;
@@ -210,6 +222,9 @@ private:
      */
     osmium::item_type m_id_type = osmium::item_type::undefined;
 
+    /// Cluster the table by geometry.
+    bool m_cluster_by_geom = true;
+
 }; // class flex_table_t
 
 class table_connection_t
@@ -255,6 +270,13 @@ public:
 
     geom::osmium_builder_t *get_builder() { return &m_builder; }
 
+    void task_set(std::future<std::chrono::milliseconds> &&future)
+    {
+        m_task_result.set(std::move(future));
+    }
+
+    void task_wait();
+
 private:
     geom::osmium_builder_t m_builder;
 
@@ -271,6 +293,8 @@ private:
     /// The connection to the database server.
     std::unique_ptr<pg_conn_t> m_db_connection;
 
+    task_result_t m_task_result;
+
     /// Has the Id index already been created?
     bool m_id_index_created = false;
 
