@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <algorithm>
 #include <cstring>
 #include <stdexcept>
@@ -8,18 +17,10 @@
 #include "domain-matcher.hpp"
 #include "format.hpp"
 #include "gazetteer-style.hpp"
+#include "logging.hpp"
 #include "pgsql.hpp"
 #include "wkb.hpp"
 
-namespace {
-
-enum : int
-{
-    MAX_ADMINLEVEL = 15
-};
-
-} // anonymous namespace
-
 namespace pt = boost::property_tree;
 
 void db_deleter_place_t::delete_rows(std::string const &table,
@@ -33,23 +34,27 @@ void db_deleter_place_t::delete_rows(std::string const &table,
     // the remainder of the SQL command.
     sql.reserve(m_deletables.size() * 43 + 200);
 
-    fmt::format_to(sql, "DELETE FROM {} p USING (VALUES ", table);
+    fmt::format_to(std::back_inserter(sql), "DELETE FROM {} p USING (VALUES ",
+                   table);
 
     for (auto const &item : m_deletables) {
-        fmt::format_to(sql, "('{}',{},", item.osm_type, item.osm_id);
+        fmt::format_to(std::back_inserter(sql), "('{}',{},", item.osm_type,
+                       item.osm_id);
         if (item.classes.empty()) {
-            fmt::format_to(sql, "ARRAY[]::text[]),");
+            fmt::format_to(std::back_inserter(sql), "ARRAY[]::text[]),");
         } else {
-            fmt::format_to(sql, "ARRAY[{}]),", item.classes);
+            fmt::format_to(std::back_inserter(sql), "ARRAY[{}]),",
+                           item.classes);
         }
     }
 
     // remove the final comma
     sql.resize(sql.size() - 1);
 
-    fmt::format_to(sql, ") AS t (osm_type, osm_id, classes) WHERE"
-                        " p.osm_type = t.osm_type AND p.osm_id = t.osm_id"
-                        " AND NOT p.class = ANY(t.classes)");
+    fmt::format_to(std::back_inserter(sql),
+                   ") AS t (osm_type, osm_id, classes) WHERE"
+                   " p.osm_type = t.osm_type AND p.osm_id = t.osm_id"
+                   " AND NOT p.class = ANY(t.classes)");
 
     conn->exec(fmt::to_string(sql));
 }
@@ -69,7 +74,8 @@ std::string gazetteer_style_t::class_list() const
     fmt::memory_buffer buf;
 
     for (auto const &m : m_main) {
-        fmt::format_to(buf, FMT_STRING("'{}',"), std::get<0>(m));
+        fmt::format_to(std::back_inserter(buf), FMT_STRING("'{}',"),
+                       std::get<0>(m));
     }
 
     if (buf.size() > 0) {
@@ -81,7 +87,7 @@ std::string gazetteer_style_t::class_list() const
 
 void gazetteer_style_t::load_style(std::string const &filename)
 {
-    fmt::print(stderr, "Parsing gazetteer style file '{}'.\n", filename);
+    log_info("Parsing gazetteer style file '{}'.", filename);
     pt::ptree root;
 
     pt::read_json(filename, root);
@@ -192,7 +198,7 @@ void gazetteer_style_t::add_style_entry(std::string const &key,
         }
         if (!value.empty()) {
             throw std::runtime_error{
-                "Style error. Prefix key can only be used with empty value.\n"};
+                "Style error. Prefix key can only be used with empty value."};
         }
         m_matcher.emplace_back(key.substr(0, key.size() - 1), flags,
                                matcher_t::MT_PREFIX);
@@ -203,7 +209,7 @@ void gazetteer_style_t::add_style_entry(std::string const &key,
     if (key[0] == '*') {
         if (!value.empty()) {
             throw std::runtime_error{
-                "Style error. Suffix key can only be used with empty value.\n"};
+                "Style error. Suffix key can only be used with empty value."};
         }
         m_matcher.emplace_back(key.substr(1), flags, matcher_t::MT_SUFFIX);
         return;
@@ -218,12 +224,12 @@ void gazetteer_style_t::add_style_entry(std::string const &key,
     if (add_metadata_style_entry(key)) {
         if (!value.empty()) {
             throw std::runtime_error{"Style error. Rules for OSM metadata "
-                                     "attributes must have an empty value.\n"};
+                                     "attributes must have an empty value."};
         }
         if (flags != SF_EXTRA) {
             throw std::runtime_error{"Style error. Rules for OSM metadata "
                                      "attributes must have the style flag "
-                                     "\"extra\" and no other flag.\n"};
+                                     "\"extra\" and no other flag."};
         }
         return;
     }
@@ -289,7 +295,7 @@ void gazetteer_style_t::process_tags(osmium::OSMObject const &o)
     bool has_postcode = false;
     bool has_country = false;
     char const *place = nullptr;
-    flag_t place_flag;
+    flag_t place_flag = 0;
     bool address_point = false;
     bool interpolation = false;
     bool admin_boundary = false;
@@ -339,7 +345,7 @@ void gazetteer_style_t::process_tags(osmium::OSMObject const &o)
         }
 
         if (flag & SF_ADDRESS) {
-            char const *addr_key;
+            char const *addr_key = nullptr;
             if (std::strncmp(k, "addr:", 5) == 0) {
                 addr_key = k + 5;
             } else if (std::strncmp(k, "is_in:", 6) == 0) {
