@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -17,6 +17,7 @@ extern "C"
 }
 
 #include <cassert>
+#include <stdexcept>
 
 // The lua_getextraspace() function is only available from Lua 5.3. For
 // earlier versions we fall back to storing the context pointer in the
@@ -125,6 +126,24 @@ char const *luaX_get_table_string(lua_State *lua_state, char const *key,
     return lua_tostring(lua_state, -1);
 }
 
+char const *luaX_get_table_string(lua_State *lua_state, char const *key,
+                                  int table_index, char const *error_msg,
+                                  char const *default_value)
+{
+    assert(lua_state);
+    assert(key);
+    lua_getfield(lua_state, table_index, key);
+    int const ltype = lua_type(lua_state, -1);
+    if (ltype == LUA_TNIL) {
+        return default_value;
+    }
+    if (ltype != LUA_TSTRING) {
+        throw std::runtime_error{
+            "{} must contain a '{}' string field."_format(error_msg, key)};
+    }
+    return lua_tostring(lua_state, -1);
+}
+
 bool luaX_get_table_bool(lua_State *lua_state, char const *key, int table_index,
                          char const *error_msg, bool default_value)
 {
