@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -24,25 +24,27 @@
 
 #include "db-copy-mgr.hpp"
 #include "middle.hpp"
-#include "node-persistent-cache.hpp"
-#include "node-ram-cache.hpp"
-#include "options.hpp"
 #include "pgsql.hpp"
 
+class node_locations_t;
+class node_persistent_cache;
+class options_t;
+
 class middle_query_pgsql_t : public middle_query_t
 {
 public:
     middle_query_pgsql_t(
         std::string const &conninfo,
-        std::shared_ptr<node_ram_cache> const &cache,
+        std::shared_ptr<node_locations_t> const &cache,
         std::shared_ptr<node_persistent_cache> const &persistent_cache);
 
     size_t nodes_get_list(osmium::WayNodeList *nodes) const override;
 
     bool way_get(osmid_t id, osmium::memory::Buffer *buffer) const override;
 
-    size_t rel_way_members_get(osmium::Relation const &rel, rolelist_t *roles,
-                               osmium::memory::Buffer *buffer) const override;
+    size_t rel_members_get(osmium::Relation const &rel,
+                           osmium::memory::Buffer *buffer,
+                           osmium::osm_entity_bits::type types) const override;
 
     bool relation_get(osmid_t id,
                       osmium::memory::Buffer *buffer) const override;
@@ -54,7 +56,7 @@ private:
     std::size_t get_way_node_locations_db(osmium::WayNodeList *nodes) const;
 
     pg_conn_t m_sql_conn;
-    std::shared_ptr<node_ram_cache> m_cache;
+    std::shared_ptr<node_locations_t> m_cache;
     std::shared_ptr<node_persistent_cache> m_persistent_cache;
 };
 
@@ -68,10 +70,13 @@ struct table_sql {
 
 struct middle_pgsql_t : public middle_t
 {
-    explicit middle_pgsql_t(options_t const *options);
+    middle_pgsql_t(std::shared_ptr<thread_pool_t> thread_pool,
+                   options_t const *options);
 
     void start() override;
-    void stop(thread_pool_t &pool) override;
+    void stop() override;
+
+    void wait() override;
 
     void node(osmium::Node const &node) override;
     void way(osmium::Way const &way) override;
@@ -114,8 +119,16 @@ struct middle_pgsql_t : public middle_t
         std::string m_prepare_fw_dep_lookups;
         std::string m_create_fw_dep_indexes;
 
+        void task_set(std::future<std::chrono::milliseconds> &&future)
+        {
+            m_task_result.set(std::move(future));
+        }
+
+        std::chrono::milliseconds task_wait() { return m_task_result.wait(); }
+
     private:
         std::shared_ptr<db_target_descr_t> m_copy_target;
+        task_result_t m_task_result;
     };
 
     std::shared_ptr<middle_query_t> get_query_instance() override;
@@ -136,7 +149,7 @@ private:
 
     options_t const *m_options;
 
-    std::shared_ptr<node_ram_cache> m_cache;
+    std::shared_ptr<node_locations_t> m_cache;
     std::shared_ptr<node_persistent_cache> m_persistent_cache;
 
     pg_conn_t m_db_connection;
