@@ -3,13 +3,11 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
-#include <algorithm>
 #include <cassert>
-#include <tuple>
 #include <vector>
 
 #include <osmium/area/geom_assembler.hpp>
