@@ -3,11 +3,13 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
+#include "db-copy.hpp"
 #include "expire-tiles.hpp"
+#include "format.hpp"
 #include "geom-transform.hpp"
 #include "logging.hpp"
 #include "lua-init.hpp"
@@ -18,6 +20,7 @@
 #include "output-flex.hpp"
 #include "pgsql.hpp"
 #include "reprojection.hpp"
+#include "thread-pool.hpp"
 #include "util.hpp"
 #include "version.hpp"
 #include "wkb.hpp"
@@ -30,8 +33,12 @@ extern "C"
 #include <lualib.h>
 }
 
+#include <rapidjson/stringbuffer.h>
+#include <rapidjson/writer.h>
+
+#include <boost/filesystem.hpp>
+
 #include <cassert>
-#include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <limits>
@@ -66,6 +73,7 @@ TRAMPOLINE(app_define_table, define_table)
 TRAMPOLINE(app_get_bbox, get_bbox)
 TRAMPOLINE(table_name, name)
 TRAMPOLINE(table_schema, schema)
+TRAMPOLINE(table_cluster, cluster)
 TRAMPOLINE(table_add_row, add_row)
 TRAMPOLINE(table_columns, columns)
 TRAMPOLINE(table_tostring, __tostring)
@@ -302,6 +310,11 @@ void write_integer(db_copy_mgr_t<db_deleter_by_type_and_id_t> *copy_mgr,
 static void write_double(db_copy_mgr_t<db_deleter_by_type_and_id_t> *copy_mgr,
                          flex_table_column_t const &column, char const *str)
 {
+    if (*str == '\0') {
+        write_null(copy_mgr, column);
+        return;
+    }
+
     char *end = nullptr;
     double const value = std::strtod(str, &end);
 
@@ -313,6 +326,142 @@ static void write_double(db_copy_mgr_t<db_deleter_by_type_and_id_t> *copy_mgr,
     copy_mgr->add_column(value);
 }
 
+using json_writer_type = rapidjson::Writer<rapidjson::StringBuffer>;
+using table_register_type = std::vector<void const *>;
+
+/**
+ * Check that the value on the top of the Lua stack is a simple array.
+ * This means that all keys must be consecutive integers starting from 1.
+ */
+static bool is_lua_array(lua_State *lua_state)
+{
+    uint32_t n = 1;
+    lua_pushnil(lua_state);
+    while (lua_next(lua_state, -2) != 0) {
+        lua_pop(lua_state, 1); // remove value from stack
+#if LUA_VERSION_NUM >= 503
+        if (!lua_isinteger(lua_state, -1)) {
+            lua_pop(lua_state, 1);
+            return false;
+        }
+        int okay = 0;
+        auto const num = lua_tointegerx(lua_state, -1, &okay);
+        if (!okay || num != n++) {
+            lua_pop(lua_state, 1);
+            return false;
+        }
+#else
+        if (!lua_isnumber(lua_state, -1)) {
+            lua_pop(lua_state, 1);
+            return false;
+        }
+        double const num = lua_tonumber(lua_state, -1);
+        double intpart = 0.0;
+        if (std::modf(num, &intpart) != 0.0 || intpart < 0 ||
+            static_cast<uint32_t>(num) != n++) {
+            lua_pop(lua_state, 1);
+            return false;
+        }
+#endif
+    }
+
+    // An empty lua table could be both, we decide here that it is not stored
+    // as a JSON array but as a JSON object.
+    return n != 1;
+}
+
+static void write_json(json_writer_type *writer, lua_State *lua_state,
+                       table_register_type *tables);
+
+static void write_json_table(json_writer_type *writer, lua_State *lua_state,
+                             table_register_type *tables)
+{
+    void const *table_ptr = lua_topointer(lua_state, -1);
+    assert(table_ptr);
+    auto const it = std::find(tables->cbegin(), tables->cend(), table_ptr);
+    if (it != tables->cend()) {
+        throw std::runtime_error{"Loop detected in table"};
+    }
+    tables->push_back(table_ptr);
+
+    if (is_lua_array(lua_state)) {
+        writer->StartArray();
+        lua_pushnil(lua_state);
+        while (lua_next(lua_state, -2) != 0) {
+            write_json(writer, lua_state, tables);
+            lua_pop(lua_state, 1);
+        }
+        writer->EndArray();
+    } else {
+        writer->StartObject();
+        lua_pushnil(lua_state);
+        while (lua_next(lua_state, -2) != 0) {
+            int const ltype_key = lua_type(lua_state, -2);
+            if (ltype_key != LUA_TSTRING) {
+                throw std::runtime_error{
+                    "Incorrect data type '{}' as key."_format(
+                        lua_typename(lua_state, ltype_key))};
+            }
+            char const *const key = lua_tostring(lua_state, -2);
+            writer->Key(key);
+            write_json(writer, lua_state, tables);
+            lua_pop(lua_state, 1);
+        }
+        writer->EndObject();
+    }
+}
+
+static void write_json_number(json_writer_type *writer, lua_State *lua_state)
+{
+#if LUA_VERSION_NUM >= 503
+    int okay = 0;
+    auto const num = lua_tointegerx(lua_state, -1, &okay);
+    if (okay) {
+        writer->Int64(num);
+    } else {
+        writer->Double(lua_tonumber(lua_state, -1));
+    }
+#else
+    double const num = lua_tonumber(lua_state, -1);
+    double intpart = 0.0;
+    if (std::modf(num, &intpart) == 0.0) {
+        writer->Int64(static_cast<int64_t>(num));
+    } else {
+        writer->Double(num);
+    }
+#endif
+}
+
+static void write_json(json_writer_type *writer, lua_State *lua_state,
+                       table_register_type *tables)
+{
+    assert(writer);
+    assert(lua_state);
+
+    int const ltype = lua_type(lua_state, -1);
+    switch (ltype) {
+    case LUA_TNIL:
+        writer->Null();
+        break;
+    case LUA_TBOOLEAN:
+        writer->Bool(lua_toboolean(lua_state, -1) != 0);
+        break;
+    case LUA_TNUMBER:
+        write_json_number(writer, lua_state);
+        break;
+    case LUA_TSTRING:
+        writer->String(lua_tostring(lua_state, -1));
+        break;
+    case LUA_TTABLE:
+        write_json_table(writer, lua_state, tables);
+        break;
+    default:
+        throw std::runtime_error{
+            "Invalid type '{}' for json/jsonb column."_format(
+                lua_typename(lua_state, ltype))};
+    }
+}
+
 void output_flex_t::write_column(
     db_copy_mgr_t<db_deleter_by_type_and_id_t> *copy_mgr,
     flex_table_column_t const &column)
@@ -342,8 +491,7 @@ void output_flex_t::write_column(
         return;
     }
 
-    if ((column.type() == table_column_type::sql) ||
-        (column.type() == table_column_type::text)) {
+    if (column.type() == table_column_type::text) {
         auto const *const str = lua_tolstring(lua_state(), -1, nullptr);
         if (!str) {
             throw std::runtime_error{
@@ -462,6 +610,13 @@ void output_flex_t::write_column(
                 "Invalid type '{}' for hstore column."_format(
                     lua_typename(lua_state(), ltype))};
         }
+    } else if ((column.type() == table_column_type::json) ||
+               (column.type() == table_column_type::jsonb)) {
+        rapidjson::StringBuffer stream;
+        json_writer_type writer{stream};
+        table_register_type tables;
+        write_json(&writer, lua_state(), &tables);
+        copy_mgr->add_column(stream.GetString());
     } else if (column.type() == table_column_type::direction) {
         switch (ltype) {
         case LUA_TBOOLEAN:
@@ -532,7 +687,7 @@ std::size_t output_flex_t::get_way_nodes()
 {
     assert(m_context_way);
     if (m_num_way_nodes == std::numeric_limits<std::size_t>::max()) {
-        m_num_way_nodes = m_mid->nodes_get_list(&m_context_way->nodes());
+        m_num_way_nodes = middle().nodes_get_list(&m_context_way->nodes());
     }
 
     return m_num_way_nodes;
@@ -616,6 +771,28 @@ flex_table_t &output_flex_t::create_flex_table()
     }
     lua_pop(lua_state(), 1);
 
+    // optional "cluster" field
+    lua_getfield(lua_state(), -1, "cluster");
+    int const cluster_type = lua_type(lua_state(), -1);
+    if (cluster_type == LUA_TSTRING) {
+        std::string const cluster = lua_tostring(lua_state(), -1);
+        if (cluster == "auto") {
+            new_table.set_cluster_by_geom(true);
+        } else if (cluster == "no") {
+            new_table.set_cluster_by_geom(false);
+        } else {
+            throw std::runtime_error{
+                "Unknown value '{}' for 'cluster' table option"
+                " (use 'auto' or 'no')."_format(cluster)};
+        }
+    } else if (cluster_type == LUA_TNIL) {
+        // ignore
+    } else {
+        throw std::runtime_error{
+            "Unknown value for 'cluster' table option: Must be string."};
+    }
+    lua_pop(lua_state(), 1);
+
     // optional "data_tablespace" field
     lua_getfield(lua_state(), -1, "data_tablespace");
     if (lua_isstring(lua_state(), -1)) {
@@ -642,8 +819,8 @@ void output_flex_t::setup_id_columns(flex_table_t *table)
     assert(table);
     lua_getfield(lua_state(), -1, "ids");
     if (lua_type(lua_state(), -1) != LUA_TTABLE) {
-        log_warn("Table '{}' doesn't have an 'ids' column. Updates"
-                 " and expire will not work!",
+        log_warn("Table '{}' doesn't have an id column. Two-stage"
+                 " processing, updates and expire will not work!",
                  table->name());
         lua_pop(lua_state(), 1); // ids
         return;
@@ -667,7 +844,7 @@ void output_flex_t::setup_id_columns(flex_table_t *table)
             std::string const column_name =
                 lua_tolstring(lua_state(), -1, nullptr);
             check_name(column_name, "column");
-            auto &column = table->add_column(column_name, "id_type");
+            auto &column = table->add_column(column_name, "id_type", "");
             column.set_not_null();
         } else if (!lua_isnil(lua_state(), -1)) {
             throw std::runtime_error{"type_column must be a string or nil."};
@@ -681,7 +858,7 @@ void output_flex_t::setup_id_columns(flex_table_t *table)
         luaX_get_table_string(lua_state(), "id_column", -2, "The ids field");
     check_name(name, "column");
 
-    auto &column = table->add_column(name, "id_num");
+    auto &column = table->add_column(name, "id_num", "");
     column.set_not_null();
     lua_pop(lua_state(), 3); // id_column, type, ids
 }
@@ -707,21 +884,23 @@ void output_flex_t::setup_flex_table_columns(flex_table_t *table)
                 "The entries in the 'columns' array must be tables."};
         }
 
-        char const *const type =
-            luaX_get_table_string(lua_state(), "type", -1, "Column entry");
+        char const *const type = luaX_get_table_string(lua_state(), "type", -1,
+                                                       "Column entry", "text");
         char const *const name =
             luaX_get_table_string(lua_state(), "column", -2, "Column entry");
         check_name(name, "column");
+        char const *const sql_type = luaX_get_table_string(
+            lua_state(), "sql_type", -3, "Column entry", "");
 
-        auto &column = table->add_column(name, type);
+        auto &column = table->add_column(name, type, sql_type);
 
-        column.set_not_null(luaX_get_table_bool(lua_state(), "not_null", -3,
+        column.set_not_null(luaX_get_table_bool(lua_state(), "not_null", -4,
                                                 "Entry 'not_null'", false));
 
         column.set_create_only(luaX_get_table_bool(
-            lua_state(), "create_only", -4, "Entry 'create_only'", false));
+            lua_state(), "create_only", -5, "Entry 'create_only'", false));
 
-        lua_getfield(lua_state(), -5, "projection");
+        lua_getfield(lua_state(), -6, "projection");
         if (!lua_isnil(lua_state(), -1)) {
             if (column.is_geometry_column() ||
                 column.type() == table_column_type::area) {
@@ -732,8 +911,9 @@ void output_flex_t::setup_flex_table_columns(flex_table_t *table)
             }
         }
 
-        // stack has: projection, create_only, not_null, column, type, table
-        lua_pop(lua_state(), 6);
+        // stack has: projection, create_only, not_null, sql_type, column,
+        //            type, table
+        lua_pop(lua_state(), 7);
         ++num_columns;
     }
 
@@ -903,6 +1083,13 @@ int output_flex_t::table_schema()
     return 1;
 }
 
+int output_flex_t::table_cluster()
+{
+    auto const &table = get_table_from_param();
+    lua_pushboolean(lua_state(), table.cluster_by_geom());
+    return 1;
+}
+
 static std::unique_ptr<geom_transform_t>
 get_transform(lua_State *lua_state, flex_table_column_t const &column)
 {
@@ -999,15 +1186,15 @@ output_flex_t::run_transform(geom::osmium_builder_t *builder,
                              osmium::Relation const &relation)
 {
     m_buffer.clear();
-    auto const num_ways =
-        m_mid->rel_way_members_get(relation, nullptr, &m_buffer);
+    auto const num_ways = middle().rel_members_get(
+        relation, &m_buffer, osmium::osm_entity_bits::way);
 
     if (num_ways == 0) {
         return {};
     }
 
     for (auto &way : m_buffer.select<osmium::Way>()) {
-        m_mid->nodes_get_list(&(way.nodes()));
+        middle().nodes_get_list(&(way.nodes()));
     }
 
     return transform->run(builder, target_geom_type, relation, m_buffer);
@@ -1048,7 +1235,7 @@ void output_flex_t::add_row(table_connection_t *table_connection,
     auto const wkbs =
         run_transform(builder, transform, table.geom_column().type(), object);
     for (auto const &wkb : wkbs) {
-        m_expire.from_wkb(wkb.c_str(), id);
+        m_expire.from_wkb(wkb, id);
         write_row(table_connection, object.type(), id, wkb,
                   table.geom_column().srid());
     }
@@ -1088,7 +1275,7 @@ void output_flex_t::pending_way(osmid_t id)
     }
 
     m_buffer.clear();
-    if (!m_mid->way_get(id, &m_buffer)) {
+    if (!middle().way_get(id, &m_buffer)) {
         return;
     }
 
@@ -1172,7 +1359,7 @@ void output_flex_t::select_relation_members(osmid_t id)
         return;
     }
 
-    if (!m_mid->relation_get(id, &m_rels_buffer)) {
+    if (!middle().relation_get(id, &m_rels_buffer)) {
         return;
     }
 
@@ -1187,7 +1374,7 @@ void output_flex_t::pending_relation(osmid_t id)
         return;
     }
 
-    if (!m_mid->relation_get(id, &m_rels_buffer)) {
+    if (!middle().relation_get(id, &m_rels_buffer)) {
         return;
     }
     auto const &relation = m_rels_buffer.get<osmium::Relation>(0);
@@ -1210,7 +1397,7 @@ void output_flex_t::pending_relation_stage1c(osmid_t id)
         return;
     }
 
-    if (!m_mid->relation_get(id, &m_rels_buffer)) {
+    if (!middle().relation_get(id, &m_rels_buffer)) {
         return;
     }
     auto const &relation = m_rels_buffer.get<osmium::Relation>(0);
@@ -1231,12 +1418,12 @@ void output_flex_t::sync()
     }
 }
 
-void output_flex_t::stop(thread_pool_t *pool)
+void output_flex_t::stop()
 {
     for (auto &table : m_table_connections) {
-        pool->submit([&]() {
-            table.stop(m_options.slim & !m_options.droptemp, m_options.append);
-        });
+        table.task_set(thread_pool().submit([&]() {
+            table.stop(m_options.slim && !m_options.droptemp, m_options.append);
+        }));
     }
 
     if (m_options.expire_tiles_zoom_min > 0) {
@@ -1245,6 +1432,13 @@ void output_flex_t::stop(thread_pool_t *pool)
     }
 }
 
+void output_flex_t::wait()
+{
+    for (auto &table : m_table_connections) {
+        table.task_wait();
+    }
+}
+
 void output_flex_t::node_add(osmium::Node const &node)
 {
     if (!m_process_node) {
@@ -1289,7 +1483,7 @@ void output_flex_t::delete_from_table(table_connection_t *table_connection,
     assert(table_connection);
     auto const id = table_connection->table().map_id(type, osm_id);
 
-    if (table_connection->table().has_geom_column()) {
+    if (m_expire.enabled() && table_connection->table().has_geom_column()) {
         auto const result = table_connection->get_geom_by_id(type, id);
 
         if (result.num_tuples() == 0) {
@@ -1305,7 +1499,7 @@ void output_flex_t::delete_from_table(table_connection_t *table_connection,
 void output_flex_t::delete_from_tables(osmium::item_type type, osmid_t osm_id)
 {
     for (auto &table : m_table_connections) {
-        if (table.table().matches_type(type)) {
+        if (table.table().matches_type(type) && table.table().has_id_column()) {
             delete_from_table(&table, type, osm_id);
         }
     }
@@ -1369,13 +1563,14 @@ output_flex_t::clone(std::shared_ptr<middle_query_t> const &mid,
                      std::shared_ptr<db_copy_thread_t> const &copy_thread) const
 {
     return std::make_shared<output_flex_t>(
-        mid, *get_options(), copy_thread, true, m_lua_state, m_process_node,
-        m_process_way, m_process_relation, m_select_relation_members, m_tables,
-        m_stage2_way_ids);
+        mid, m_thread_pool, *get_options(), copy_thread, true, m_lua_state,
+        m_process_node, m_process_way, m_process_relation,
+        m_select_relation_members, m_tables, m_stage2_way_ids);
 }
 
 output_flex_t::output_flex_t(
-    std::shared_ptr<middle_query_t> const &mid, options_t const &o,
+    std::shared_ptr<middle_query_t> const &mid,
+    std::shared_ptr<thread_pool_t> thread_pool, options_t const &o,
     std::shared_ptr<db_copy_thread_t> const &copy_thread, bool is_clone,
     std::shared_ptr<lua_State> lua_state, prepared_lua_function_t process_node,
     prepared_lua_function_t process_way,
@@ -1383,7 +1578,7 @@ output_flex_t::output_flex_t(
     prepared_lua_function_t select_relation_members,
     std::shared_ptr<std::vector<flex_table_t>> tables,
     std::shared_ptr<idset_t> stage2_way_ids)
-: output_t(mid, o), m_tables(std::move(tables)),
+: output_t(mid, std::move(thread_pool), o), m_tables(std::move(tables)),
   m_stage2_way_ids(std::move(stage2_way_ids)), m_copy_thread(copy_thread),
   m_lua_state(std::move(lua_state)),
   m_expire(o.expire_tiles_zoom, o.expire_tiles_max_bbox, o.projection),
@@ -1397,6 +1592,13 @@ output_flex_t::output_flex_t(
 
     if (!is_clone) {
         init_lua(m_options.style);
+
+        // If the osm2pgsql.select_relation_members() Lua function is defined
+        // it means we need two-stage processing which in turn means we need
+        // the full ways stored in the middle.
+        if (m_select_relation_members) {
+            m_output_requirements.full_ways = true;
+        }
     }
 
     if (m_tables->empty()) {
@@ -1430,6 +1632,10 @@ void output_flex_t::init_lua(std::string const &filename)
                        m_options.append ? "append" : "create");
     luaX_add_table_int(lua_state(), "stage", 1);
 
+    std::string const dir_path =
+        boost::filesystem::path{filename}.parent_path().string();
+    luaX_add_table_str(lua_state(), "config_dir", dir_path.c_str());
+
     luaX_add_table_func(lua_state(), "define_table",
                         lua_trampoline_app_define_table);
 
@@ -1446,6 +1652,7 @@ void output_flex_t::init_lua(std::string const &filename)
     luaX_add_table_func(lua_state(), "add_row", lua_trampoline_table_add_row);
     luaX_add_table_func(lua_state(), "name", lua_trampoline_table_name);
     luaX_add_table_func(lua_state(), "schema", lua_trampoline_table_schema);
+    luaX_add_table_func(lua_state(), "cluster", lua_trampoline_table_cluster);
     luaX_add_table_func(lua_state(), "columns", lua_trampoline_table_columns);
 
     // Clean up stack
@@ -1522,7 +1729,8 @@ void output_flex_t::reprocess_marked()
         util::timer_t timer;
 
         for (auto &table : m_table_connections) {
-            if (table.table().matches_type(osmium::item_type::way)) {
+            if (table.table().matches_type(osmium::item_type::way) &&
+                table.table().has_id_column()) {
                 table.create_id_index();
             }
         }
@@ -1540,13 +1748,14 @@ void output_flex_t::reprocess_marked()
     lua_setfield(lua_state(), -2, "stage");
     lua_pop(lua_state(), 1); // osm2pgsql
 
+    m_stage2_way_ids->sort_unique();
+
     log_info(
         "There are {} ways to reprocess..."_format(m_stage2_way_ids->size()));
 
-    m_stage2_way_ids->sort_unique();
     for (osmid_t const id : *m_stage2_way_ids) {
         m_buffer.clear();
-        if (!m_mid->way_get(id, &m_buffer)) {
+        if (!middle().way_get(id, &m_buffer)) {
             continue;
         }
         auto &way = m_buffer.get<osmium::Way>(0);
