@@ -6,20 +6,15 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
-#include "db-copy.hpp"
 #include "expire-tiles.hpp"
 #include "flex-table-column.hpp"
 #include "flex-table.hpp"
-#include "format.hpp"
-#include "geom-transform.hpp"
 #include "osmium-builder.hpp"
 #include "output.hpp"
-#include "table.hpp"
-#include "tagtransform.hpp"
 
 #include <osmium/index/id_set.hpp>
 #include <osmium/osm/item_type.hpp>
@@ -35,6 +30,12 @@ extern "C"
 #include <utility>
 #include <vector>
 
+class db_copy_thread_t;
+class db_deleter_by_type_and_id_t;
+class geom_transform_t;
+class options_t;
+class thread_pool_t;
+
 using idset_t = osmium::index::IdSetSmall<osmid_t>;
 
 /**
@@ -99,7 +100,8 @@ class output_flex_t : public output_t
 
 public:
     output_flex_t(
-        std::shared_ptr<middle_query_t> const &mid, options_t const &options,
+        std::shared_ptr<middle_query_t> const &mid,
+        std::shared_ptr<thread_pool_t> thread_pool, options_t const &options,
         std::shared_ptr<db_copy_thread_t> const &copy_thread,
         bool is_clone = false, std::shared_ptr<lua_State> lua_state = nullptr,
         prepared_lua_function_t process_node = {},
@@ -123,9 +125,11 @@ public:
           std::shared_ptr<db_copy_thread_t> const &copy_thread) const override;
 
     void start() override;
-    void stop(thread_pool_t *pool) override;
+    void stop() override;
     void sync() override;
 
+    void wait() override;
+
     idset_t const &get_marked_way_ids() override;
     void reprocess_marked() override;
 
@@ -157,6 +161,7 @@ public:
     int table_add_row();
     int table_name();
     int table_schema();
+    int table_cluster();
     int table_columns();
 
 private:
