@@ -6,35 +6,41 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
 #include <memory>
+#include <utility>
 
 #include <osmium/memory/buffer.hpp>
 
-#include "db-copy-mgr.hpp"
 #include "gazetteer-style.hpp"
 #include "osmium-builder.hpp"
 #include "osmtypes.hpp"
 #include "output.hpp"
 
+class db_copy_thread_t;
+class thread_pool_t;
+
+struct middle_query_t;
+
 class output_gazetteer_t : public output_t
 {
     output_gazetteer_t(output_gazetteer_t const *other,
                        std::shared_ptr<middle_query_t> const &cloned_mid,
                        std::shared_ptr<db_copy_thread_t> const &copy_thread)
-    : output_t(cloned_mid, other->m_options), m_copy(copy_thread),
-      m_builder(other->m_options.projection),
+    : output_t(cloned_mid, other->m_thread_pool, other->m_options),
+      m_copy(copy_thread), m_builder(other->m_options.projection),
       m_osmium_buffer(PLACE_BUFFER_SIZE, osmium::memory::Buffer::auto_grow::yes)
     {}
 
 public:
     output_gazetteer_t(std::shared_ptr<middle_query_t> const &mid,
+                       std::shared_ptr<thread_pool_t> thread_pool,
                        options_t const &options,
                        std::shared_ptr<db_copy_thread_t> const &copy_thread)
-    : output_t(mid, options), m_copy(copy_thread),
+    : output_t(mid, std::move(thread_pool), options), m_copy(copy_thread),
       m_builder(options.projection),
       m_osmium_buffer(PLACE_BUFFER_SIZE, osmium::memory::Buffer::auto_grow::yes)
     {
@@ -50,7 +56,7 @@ public:
     }
 
     void start() override;
-    void stop(thread_pool_t *) noexcept override {}
+    void stop() noexcept override {}
     void sync() override;
 
     void pending_way(osmid_t) noexcept override {}
