@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -32,7 +32,6 @@
 #include "expire-tiles.hpp"
 #include "logging.hpp"
 #include "middle.hpp"
-#include "node-ram-cache.hpp"
 #include "options.hpp"
 #include "osmtypes.hpp"
 #include "output-pgsql.hpp"
@@ -50,7 +49,7 @@ void output_pgsql_t::pgsql_out_way(osmium::Way const &way, taglist_t *tags,
     if (polygon && way.is_closed()) {
         auto wkb = m_builder.get_wkb_polygon(way);
         if (!wkb.empty()) {
-            expire.from_wkb(wkb.c_str(), way.id());
+            m_expire.from_wkb(wkb, way.id());
             if (m_enable_way_area) {
                 auto const area =
                     m_options.reproject_area
@@ -67,7 +66,7 @@ void output_pgsql_t::pgsql_out_way(osmium::Way const &way, taglist_t *tags,
         double const split_at =
             m_options.projection->target_latlon() ? 1 : 100 * 1000;
         for (auto const &wkb : m_builder.get_wkb_line(way.nodes(), split_at)) {
-            expire.from_wkb(wkb.c_str(), way.id());
+            m_expire.from_wkb(wkb, way.id());
             m_tables[t_line]->write_row(way.id(), *tags, wkb);
             if (roads) {
                 m_tables[t_roads]->write_row(way.id(), *tags, wkb);
@@ -79,16 +78,16 @@ void output_pgsql_t::pgsql_out_way(osmium::Way const &way, taglist_t *tags,
 void output_pgsql_t::pending_way(osmid_t id)
 {
     // Try to fetch the way from the DB
-    buffer.clear();
-    if (m_mid->way_get(id, &buffer)) {
+    m_buffer.clear();
+    if (middle().way_get(id, &m_buffer)) {
         pgsql_delete_way_from_output(id);
 
         taglist_t outtags;
-        int polygon = 0;
-        int roads = 0;
-        auto &way = buffer.get<osmium::Way>(0);
+        bool polygon = false;
+        bool roads = false;
+        auto &way = m_buffer.get<osmium::Way>(0);
         if (!m_tagtransform->filter_tags(way, &polygon, &roads, outtags)) {
-            auto nnodes = m_mid->nodes_get_list(&(way.nodes()));
+            auto nnodes = middle().nodes_get_list(&(way.nodes()));
             if (nnodes > 1) {
                 pgsql_out_way(way, &outtags, polygon, roads);
                 return;
@@ -103,11 +102,11 @@ void output_pgsql_t::pending_relation(osmid_t id)
     // Note that we cannot use the global buffer here because
     // we cannot keep a reference to the relation and an autogrow buffer
     // might be relocated when more data is added.
-    rels_buffer.clear();
-    if (m_mid->relation_get(id, &rels_buffer)) {
+    m_rels_buffer.clear();
+    if (middle().relation_get(id, &m_rels_buffer)) {
         pgsql_delete_relation_from_output(id);
 
-        auto const &rel = rels_buffer.get<osmium::Relation>(0);
+        auto const &rel = m_rels_buffer.get<osmium::Relation>(0);
         pgsql_process_relation(rel);
     }
 }
@@ -119,19 +118,25 @@ void output_pgsql_t::sync()
     }
 }
 
-void output_pgsql_t::stop(thread_pool_t *pool)
+void output_pgsql_t::stop()
 {
-    // attempt to stop tables in parallel
     for (auto &t : m_tables) {
-        pool->submit([&]() {
-            t->stop(m_options.slim & !m_options.droptemp,
+        t->task_set(thread_pool().submit([&]() {
+            t->stop(m_options.slim && !m_options.droptemp,
                     m_options.enable_hstore_index, m_options.tblsmain_index);
-        });
+        }));
     }
 
     if (m_options.expire_tiles_zoom_min > 0) {
-        expire.output_and_destroy(m_options.expire_tiles_filename.c_str(),
-                                  m_options.expire_tiles_zoom_min);
+        m_expire.output_and_destroy(m_options.expire_tiles_filename.c_str(),
+                                    m_options.expire_tiles_zoom_min);
+    }
+}
+
+void output_pgsql_t::wait()
+{
+    for (auto &t : m_tables) {
+        t->task_wait();
     }
 }
 
@@ -143,14 +148,14 @@ void output_pgsql_t::node_add(osmium::Node const &node)
     }
 
     auto wkb = m_builder.get_wkb_node(node.location());
-    expire.from_wkb(wkb.c_str(), node.id());
+    m_expire.from_wkb(wkb, node.id());
     m_tables[t_point]->write_row(node.id(), outtags, wkb);
 }
 
 void output_pgsql_t::way_add(osmium::Way *way)
 {
-    int polygon = 0;
-    int roads = 0;
+    bool polygon = false;
+    bool roads = false;
     taglist_t outtags;
 
     /* Check whether the way is: (1) Exportable, (2) Maybe a polygon */
@@ -158,13 +163,42 @@ void output_pgsql_t::way_add(osmium::Way *way)
 
     if (!filter) {
         /* Get actual node data and generate output */
-        auto nnodes = m_mid->nodes_get_list(&(way->nodes()));
+        auto nnodes = middle().nodes_get_list(&(way->nodes()));
         if (nnodes > 1) {
             pgsql_out_way(*way, &outtags, polygon, roads);
         }
     }
 }
 
+// The roles of all available member ways of a relation are available in the
+// Lua "filter_tags_relation_member" callback function. This function extracts
+// the roles from all ways in the buffer and returns the list.
+static rolelist_t get_rolelist(osmium::Relation const &rel,
+                               osmium::memory::Buffer const &buffer)
+{
+    rolelist_t roles;
+
+    auto it = buffer.select<osmium::Way>().cbegin();
+    auto const end = buffer.select<osmium::Way>().cend();
+
+    if (it == end) {
+        return roles;
+    }
+
+    for (auto const &member : rel.members()) {
+        if (member.type() == osmium::item_type::way &&
+            member.ref() == it->id()) {
+            roles.emplace_back(member.role());
+            ++it;
+            if (it == end) {
+                break;
+            }
+        }
+    }
+
+    return roles;
+}
+
 /* This is the workhorse of pgsql_add_relation, split out because it is used as the callback for iterate relations */
 void output_pgsql_t::pgsql_process_relation(osmium::Relation const &rel)
 {
@@ -173,29 +207,34 @@ void output_pgsql_t::pgsql_process_relation(osmium::Relation const &rel)
         return;
     }
 
-    buffer.clear();
-    rolelist_t xrole;
-    auto num_ways = m_mid->rel_way_members_get(rel, &xrole, &buffer);
+    m_buffer.clear();
+    auto const num_ways =
+        middle().rel_members_get(rel, &m_buffer, osmium::osm_entity_bits::way);
 
     if (num_ways == 0) {
         return;
     }
 
-    int roads = 0;
-    int make_polygon = 0;
-    int make_boundary = 0;
+    bool roads = false;
+    bool make_polygon = false;
+    bool make_boundary = false;
     taglist_t outtags;
 
+    rolelist_t xrole;
+    if (!m_options.tag_transform_script.empty()) {
+        xrole = get_rolelist(rel, m_buffer);
+    }
+
     // If it's a route relation make_boundary and make_polygon will be false
     // otherwise one or the other will be true.
-    if (m_tagtransform->filter_rel_member_tags(prefiltered_tags, buffer, xrole,
-                                               &make_boundary, &make_polygon,
-                                               &roads, outtags)) {
+    if (m_tagtransform->filter_rel_member_tags(
+            prefiltered_tags, m_buffer, xrole, &make_boundary, &make_polygon,
+            &roads, outtags)) {
         return;
     }
 
-    for (auto &w : buffer.select<osmium::Way>()) {
-        m_mid->nodes_get_list(&(w.nodes()));
+    for (auto &w : m_buffer.select<osmium::Way>()) {
+        middle().nodes_get_list(&(w.nodes()));
     }
 
     // linear features and boundaries
@@ -204,9 +243,9 @@ void output_pgsql_t::pgsql_process_relation(osmium::Relation const &rel)
     if (!make_polygon) {
         double const split_at =
             m_options.projection->target_latlon() ? 1 : 100 * 1000;
-        auto wkbs = m_builder.get_wkb_multiline(buffer, split_at);
+        auto wkbs = m_builder.get_wkb_multiline(m_buffer, split_at);
         for (auto const &wkb : wkbs) {
-            expire.from_wkb(wkb.c_str(), -rel.id());
+            m_expire.from_wkb(wkb, -rel.id());
             m_tables[t_line]->write_row(-rel.id(), outtags, wkb);
             if (roads) {
                 m_tables[t_roads]->write_row(-rel.id(), outtags, wkb);
@@ -216,10 +255,11 @@ void output_pgsql_t::pgsql_process_relation(osmium::Relation const &rel)
 
     // multipolygons and boundaries
     if (make_boundary || make_polygon) {
-        auto wkbs = m_builder.get_wkb_multipolygon(rel, buffer, m_options.enable_multi);
+        auto wkbs = m_builder.get_wkb_multipolygon(rel, m_buffer,
+                                                   m_options.enable_multi);
 
         for (auto const &wkb : wkbs) {
-            expire.from_wkb(wkb.c_str(), -rel.id());
+            m_expire.from_wkb(wkb, -rel.id());
             if (m_enable_way_area) {
                 auto const area =
                     m_options.reproject_area
@@ -259,7 +299,7 @@ void output_pgsql_t::relation_add(osmium::Relation const &rel)
  * contain the change for that also. */
 void output_pgsql_t::node_delete(osmid_t osm_id)
 {
-    if (expire.from_db(m_tables[t_point].get(), osm_id) != 0) {
+    if (m_expire.from_result(m_tables[t_point]->get_wkb(osm_id), osm_id) != 0) {
         m_tables[t_point]->delete_row(osm_id);
     }
 }
@@ -277,10 +317,10 @@ void output_pgsql_t::pgsql_delete_way_from_output(osmid_t osm_id)
     }
 
     m_tables[t_roads]->delete_row(osm_id);
-    if (expire.from_db(m_tables[t_line].get(), osm_id) != 0) {
+    if (m_expire.from_result(m_tables[t_line]->get_wkb(osm_id), osm_id) != 0) {
         m_tables[t_line]->delete_row(osm_id);
     }
-    if (expire.from_db(m_tables[t_poly].get(), osm_id) != 0) {
+    if (m_expire.from_result(m_tables[t_poly]->get_wkb(osm_id), osm_id) != 0) {
         m_tables[t_poly]->delete_row(osm_id);
     }
 }
@@ -294,10 +334,12 @@ void output_pgsql_t::way_delete(osmid_t osm_id)
 void output_pgsql_t::pgsql_delete_relation_from_output(osmid_t osm_id)
 {
     m_tables[t_roads]->delete_row(-osm_id);
-    if (expire.from_db(m_tables[t_line].get(), -osm_id) != 0) {
+    if (m_expire.from_result(m_tables[t_line]->get_wkb(-osm_id), -osm_id) !=
+        0) {
         m_tables[t_line]->delete_row(-osm_id);
     }
-    if (expire.from_db(m_tables[t_poly].get(), -osm_id) != 0) {
+    if (m_expire.from_result(m_tables[t_poly]->get_wkb(-osm_id), -osm_id) !=
+        0) {
         m_tables[t_poly]->delete_row(-osm_id);
     }
 }
@@ -346,12 +388,13 @@ std::shared_ptr<output_t> output_pgsql_t::clone(
 }
 
 output_pgsql_t::output_pgsql_t(
-    std::shared_ptr<middle_query_t> const &mid, options_t const &o,
+    std::shared_ptr<middle_query_t> const &mid,
+    std::shared_ptr<thread_pool_t> thread_pool, options_t const &o,
     std::shared_ptr<db_copy_thread_t> const &copy_thread)
-: output_t(mid, o), m_builder(o.projection),
-  expire(o.expire_tiles_zoom, o.expire_tiles_max_bbox, o.projection),
-  buffer(32768, osmium::memory::Buffer::auto_grow::yes),
-  rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
+: output_t(mid, std::move(thread_pool), o), m_builder(o.projection),
+  m_expire(o.expire_tiles_zoom, o.expire_tiles_max_bbox, o.projection),
+  m_buffer(32768, osmium::memory::Buffer::auto_grow::yes),
+  m_rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
 {
     log_debug("Using projection SRS {} ({})", o.projection->target_srs(),
               o.projection->target_desc());
@@ -394,30 +437,28 @@ output_pgsql_t::output_pgsql_t(
             std::abort(); // should never be here
         }
 
-        m_tables[i].reset(
-            new table_t{name, type, columns, m_options.hstore_columns,
-                        m_options.projection->target_srs(), m_options.append,
-                        m_options.hstore_mode, copy_thread,
-                        m_options.output_dbschema});
+        m_tables[i] = std::make_unique<table_t>(
+            name, type, columns, m_options.hstore_columns,
+            m_options.projection->target_srs(), m_options.append,
+            m_options.hstore_mode, copy_thread, m_options.output_dbschema);
     }
 }
 
 output_pgsql_t::output_pgsql_t(
     output_pgsql_t const *other, std::shared_ptr<middle_query_t> const &mid,
     std::shared_ptr<db_copy_thread_t> const &copy_thread)
-: output_t(mid, other->m_options),
+: output_t(mid, other->m_thread_pool, other->m_options),
   m_tagtransform(other->m_tagtransform->clone()),
-  m_enable_way_area(other->m_enable_way_area),
-  m_builder(m_options.projection),
-  expire(m_options.expire_tiles_zoom, m_options.expire_tiles_max_bbox,
-         m_options.projection),
-  buffer(1024, osmium::memory::Buffer::auto_grow::yes),
-  rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
+  m_enable_way_area(other->m_enable_way_area), m_builder(m_options.projection),
+  m_expire(m_options.expire_tiles_zoom, m_options.expire_tiles_max_bbox,
+           m_options.projection),
+  m_buffer(1024, osmium::memory::Buffer::auto_grow::yes),
+  m_rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
 {
     for (size_t i = 0; i < t_MAX; ++i) {
         //copy constructor will just connect to the already there table
-        m_tables[i].reset(
-            new table_t{*(other->m_tables[i].get()), copy_thread});
+        m_tables[i] =
+            std::make_unique<table_t>(*(other->m_tables[i].get()), copy_thread);
     }
 }
 
@@ -427,6 +468,6 @@ void output_pgsql_t::merge_expire_trees(output_t *other)
 {
     auto *const opgsql = dynamic_cast<output_pgsql_t *>(other);
     if (opgsql) {
-        expire.merge_and_destroy(opgsql->expire);
+        m_expire.merge_and_destroy(opgsql->m_expire);
     }
 }
