@@ -1,6 +1,15 @@
 #ifndef OSM2PGSQL_OUTPUT_PGSQL_HPP
 #define OSM2PGSQL_OUTPUT_PGSQL_HPP
 
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 /* Implements the output-layer processing for osm2pgsql
  * storing the data in several PostgreSQL tables
  * with the final PostGIS geometries for each entity
@@ -8,7 +17,6 @@
 
 #include "db-copy.hpp"
 #include "expire-tiles.hpp"
-#include "id-tracker.hpp"
 #include "osmium-builder.hpp"
 #include "output.hpp"
 #include "table.hpp"
@@ -34,18 +42,22 @@ public:
     };
 
     output_pgsql_t(std::shared_ptr<middle_query_t> const &mid,
+                   std::shared_ptr<thread_pool_t> thread_pool,
                    options_t const &options,
                    std::shared_ptr<db_copy_thread_t> const &copy_thread);
-    virtual ~output_pgsql_t();
+
+    ~output_pgsql_t() override;
 
     std::shared_ptr<output_t>
     clone(std::shared_ptr<middle_query_t> const &mid,
           std::shared_ptr<db_copy_thread_t> const &copy_thread) const override;
 
     void start() override;
-    void stop(thread_pool_t *pool) override;
+    void stop() override;
     void sync() override;
 
+    void wait() override;
+
     void pending_way(osmid_t id) override;
     void pending_relation(osmid_t id) override;
 
@@ -73,15 +85,15 @@ protected:
     std::unique_ptr<tagtransform_t> m_tagtransform;
 
     //enable output of a generated way_area tag to either hstore or its own column
-    int m_enable_way_area;
+    bool m_enable_way_area;
 
     std::array<std::unique_ptr<table_t>, t_MAX> m_tables;
 
     geom::osmium_builder_t m_builder;
-    expire_tiles expire;
+    expire_tiles m_expire;
 
-    osmium::memory::Buffer buffer;
-    osmium::memory::Buffer rels_buffer;
+    osmium::memory::Buffer m_buffer;
+    osmium::memory::Buffer m_rels_buffer;
 };
 
 #endif // OSM2PGSQL_OUTPUT_PGSQL_HPP
