@@ -6,13 +6,17 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
+#include "osmtypes.hpp"
+
 #include <string>
 
-#include "pgsql.hpp"
+class pg_conn_t;
+class pg_result_t;
+
 
 /**
  * Iterate over the result from a pgsql query and generate a list of all the
@@ -31,6 +35,10 @@ void create_geom_check_trigger(pg_conn_t *db_connection,
                                std::string const &table,
                                std::string const &geom_column);
 
+void drop_geom_check_trigger(pg_conn_t *db_connection,
+                             std::string const &schema,
+                             std::string const &table);
+
 void analyze_table(pg_conn_t const &db_connection, std::string const &schema,
                    std::string const &name);
 
