@@ -3,16 +3,16 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
 #include <algorithm>
+#include <cstdint>
 #include <cstdio>
-#include <cstring>
+#include <limits>
 #include <stdexcept>
 #include <string>
-#include <utility>
 #include <vector>
 
 #include "format.hpp"
@@ -22,7 +22,6 @@
 #include "table.hpp"
 #include "taginfo.hpp"
 #include "util.hpp"
-#include "wkb.hpp"
 
 table_t::table_t(std::string const &name, std::string const &type,
                  columns_t const &columns, hstores_t const &hstore_columns,
@@ -68,7 +67,7 @@ void table_t::sync() { m_copy.sync(); }
 
 void table_t::connect()
 {
-    m_sql_conn.reset(new pg_conn_t{m_conninfo});
+    m_sql_conn = std::make_unique<pg_conn_t>(m_conninfo);
     //let commits happen faster by delaying when they actually occur
     m_sql_conn->exec("SET synchronous_commit = off");
 }
@@ -132,6 +131,11 @@ void table_t::start(std::string const &conninfo, std::string const &table_space)
 
         //create the table
         m_sql_conn->exec(sql);
+
+        if (m_srid != "4326") {
+            create_geom_check_trigger(m_sql_conn.get(), m_target->schema,
+                                      m_target->name, "way");
+        }
     }
 
     prepare();
@@ -183,7 +187,10 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
         m_target->schema, m_target->name + "_tmp");
 
     if (!m_append) {
-        util::timer_t timer;
+        if (m_srid != "4326") {
+            drop_geom_check_trigger(m_sql_conn.get(), m_target->schema,
+                                    m_target->name);
+        }
 
         log_info("Clustering table '{}' by geometry...", m_target->name);
 
@@ -195,13 +202,6 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
             "CREATE TABLE {} {} AS SELECT * FROM {}"_format(
                 qual_tmp_name, m_table_space, qual_name);
 
-        if (m_srid != "4326") {
-            // libosmium assures validity of geometries in 4326.
-            // Transformation to another projection could make the geometry
-            // invalid. Therefore add a filter to drop those.
-            sql += " WHERE ST_IsValid(way)";
-        }
-
         auto const postgis_version = get_postgis_version(*m_sql_conn);
 
         sql += " ORDER BY ";
@@ -225,8 +225,8 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
         m_sql_conn->exec(sql);
 
         m_sql_conn->exec("DROP TABLE {}"_format(qual_name));
-        m_sql_conn->exec(
-            "ALTER TABLE {} RENAME TO {}"_format(qual_tmp_name, m_target->name));
+        m_sql_conn->exec("ALTER TABLE {} RENAME TO \"{}\""_format(
+            qual_tmp_name, m_target->name));
 
         log_info("Creating geometry index on table '{}'...", m_target->name);
 
@@ -265,9 +265,6 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
         }
         log_info("Analyzing table '{}'...", m_target->name);
         analyze_table(*m_sql_conn, m_target->schema, m_target->name);
-
-        log_info("All postprocessing on table '{}' done in {}.", m_target->name,
-                 util::human_readable_duration(timer.stop()));
     }
     teardown();
 }
@@ -375,13 +372,21 @@ void table_t::write_hstore_columns(taglist_t const &tags)
     }
 }
 
+void table_t::task_wait()
+{
+    auto const run_time = m_task_result.wait();
+    log_info("All postprocessing on table '{}' done in {}.", m_target->name,
+             util::human_readable_duration(run_time));
+}
+
 /* Escape data appropriate to the type */
 void table_t::escape_type(std::string const &value, ColumnType flags)
 {
     switch (flags) {
     case ColumnType::INT: {
         // For integers we take the first number, or the average if it's a-b
-        long long from, to;
+        long long from = 0;
+        long long to = 0;
         // limit number of digits parsed to avoid undefined behaviour in sscanf
         int const items =
             std::sscanf(value.c_str(), "%18lld-%18lld", &from, &to);
@@ -415,7 +420,8 @@ void table_t::escape_type(std::string const &value, ColumnType flags)
             std::string escaped{value};
             std::replace(escaped.begin(), escaped.end(), ',', '.');
 
-            double from, to;
+            double from = NAN;
+            double to = NAN;
             int const items =
                 std::sscanf(escaped.c_str(), "%lf-%lf", &from, &to);
             if (items == 1) {
@@ -442,8 +448,8 @@ void table_t::escape_type(std::string const &value, ColumnType flags)
     }
 }
 
-table_t::wkb_reader table_t::get_wkb_reader(osmid_t id)
+pg_result_t table_t::get_wkb(osmid_t id)
 {
-    auto res = m_sql_conn->exec_prepared("get_wkb", id);
-    return wkb_reader{std::move(res)};
+    return m_sql_conn->exec_prepared("get_wkb", id);
 }
+
