@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -14,6 +14,7 @@
 #include "osmtypes.hpp"
 #include "pgsql.hpp"
 #include "taginfo.hpp"
+#include "thread-pool.hpp"
 
 #include <cstddef>
 #include <memory>
@@ -40,36 +41,19 @@ public:
 
     void sync();
 
+    void task_set(std::future<std::chrono::milliseconds> &&future)
+    {
+        m_task_result.set(std::move(future));
+    }
+
+    void task_wait();
+
     void write_row(osmid_t id, taglist_t const &tags, std::string const &geom);
     void delete_row(osmid_t id);
 
-    // interface for retrieving well known binary geometry from the table
-    class wkb_reader
-    {
-        friend table_t;
+    pg_result_t get_wkb(osmid_t id);
 
-    public:
-        char const *get_next()
-        {
-            if (m_current < m_count) {
-                return m_result.get_value(m_current++, 0);
-            }
-            return nullptr;
-        }
-
-        int get_count() const noexcept { return m_count; }
-
-    private:
-        explicit wkb_reader(pg_result_t &&result)
-        : m_result(std::move(result)), m_count(m_result.num_tuples()),
-          m_current(0)
-        {}
-
-        pg_result_t m_result;
-        int m_count;
-        int m_current;
-    };
-    wkb_reader get_wkb_reader(osmid_t id);
+    task_result_t m_task_result;
 
 protected:
     void connect();
