@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -29,17 +29,6 @@ std::vector<taginfo> const &export_list::get(osmium::item_type type) const
     return m_export_list(type);
 }
 
-bool export_list::has_column(osmium::item_type type, char const *name) const
-{
-    for (auto const &info : m_export_list(type)) {
-        if (info.name == name) {
-            return true;
-        }
-    }
-
-    return false;
-}
-
 columns_t export_list::normal_columns(osmium::item_type type) const
 {
     columns_t columns;
@@ -95,10 +84,6 @@ static unsigned get_tag_type(std::string const &tag)
 
 bool read_style_file(std::string const &filename, export_list *exlist)
 {
-    char osmtype[24];
-    char tag[64];
-    char datatype[24];
-    char flags[128];
     bool enable_way_area = true;
 
     FILE *const in = std::fopen(filename.c_str(), "rt");
@@ -121,6 +106,10 @@ bool read_style_file(std::string const &filename, export_list *exlist)
         }
 
         //grab the expected fields for this row
+        char osmtype[24] = {0};
+        char tag[64] = {0};
+        char datatype[24] = {0};
+        char flags[128] = {0};
         int const fields = std::sscanf(buffer, "%23s %63s %23s %127s", osmtype,
                                        tag, datatype, flags);
         if (fields <= 0) { /* Blank line */
@@ -135,7 +124,7 @@ bool read_style_file(std::string const &filename, export_list *exlist)
         }
 
         //place to keep info about this tag
-        taginfo temp;
+        taginfo temp{};
         temp.name = tag;
         temp.type = datatype;
         temp.flags = parse_tag_flags(flags, lineno);
