@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -20,19 +20,18 @@ public:
 
     std::unique_ptr<tagtransform_t> clone() const override;
 
-    bool filter_tags(osmium::OSMObject const &o, int *polygon, int *roads,
-                     taglist_t &out_tags, bool strict = false) override;
+    bool filter_tags(osmium::OSMObject const &o, bool *polygon, bool *roads,
+                     taglist_t &out_tags) override;
 
     bool filter_rel_member_tags(taglist_t const &rel_tags,
                                 osmium::memory::Buffer const &members,
                                 rolelist_t const &member_roles,
-                                int *make_boundary, int *make_polygon,
-                                int *roads, taglist_t &out_tags,
-                                bool allow_typeless = false) override;
+                                bool *make_boundary, bool *make_polygon,
+                                bool *roads, taglist_t &out_tags) override;
 
 private:
     bool check_key(std::vector<taginfo> const &infos, char const *k,
-                   bool *filter, int *flags, bool strict);
+                   bool *filter, unsigned int *flags);
 
     options_t const *m_options;
     export_list m_export_list;
