@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -18,14 +18,10 @@ extern "C"
 #include "tagtransform-lua.hpp"
 
 lua_tagtransform_t::lua_tagtransform_t(options_t const *options)
-: m_node_func(
-      options->tag_transform_node_func.get_value_or("filter_tags_node")),
-  m_way_func(options->tag_transform_way_func.get_value_or("filter_tags_way")),
-  m_rel_func(
-      options->tag_transform_rel_func.get_value_or("filter_basic_tags_rel")),
-  m_rel_mem_func(options->tag_transform_rel_mem_func.get_value_or(
-      "filter_tags_relation_member")),
-  m_lua_file(options->tag_transform_script.get()),
+: m_node_func("filter_tags_node"), m_way_func("filter_tags_way"),
+  m_rel_func("filter_basic_tags_rel"),
+  m_rel_mem_func("filter_tags_relation_member"),
+  m_lua_file(options->tag_transform_script),
   m_extra_attributes(options->extra_attributes)
 {
     open_style();
@@ -67,8 +63,8 @@ void lua_tagtransform_t::check_lua_function_exists(std::string const &func_name)
     lua_pop(L, 1);
 }
 
-bool lua_tagtransform_t::filter_tags(osmium::OSMObject const &o, int *polygon,
-                                     int *roads, taglist_t &out_tags, bool)
+bool lua_tagtransform_t::filter_tags(osmium::OSMObject const &o, bool *polygon,
+                                     bool *roads, taglist_t &out_tags)
 {
     switch (o.type()) {
     case osmium::item_type::node:
@@ -154,8 +150,8 @@ bool lua_tagtransform_t::filter_tags(osmium::OSMObject const &o, int *polygon,
 
 bool lua_tagtransform_t::filter_rel_member_tags(
     taglist_t const &rel_tags, osmium::memory::Buffer const &members,
-    rolelist_t const &member_roles, int *make_boundary, int *make_polygon,
-    int *roads, taglist_t &out_tags, bool)
+    rolelist_t const &member_roles, bool *make_boundary, bool *make_polygon,
+    bool *roads, taglist_t &out_tags)
 {
     size_t const num_members = member_roles.size();
     lua_getglobal(L, m_rel_mem_func.c_str());
