@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -14,6 +14,8 @@
 #include "osmtypes.hpp"
 
 #include <array>
+#include <cassert>
+#include <chrono>
 #include <cstdint>
 #include <cstdlib>
 #include <ctime>
@@ -39,7 +41,7 @@ public:
     char const *c_str() const noexcept { return m_buffer.data(); }
 
 private:
-    std::array<char, buffer_size> m_buffer;
+    std::array<char, buffer_size> m_buffer{};
 };
 
 class double_to_buffer
@@ -58,7 +60,7 @@ public:
     char const *c_str() const noexcept { return m_buffer.data(); }
 
 private:
-    std::array<char, buffer_size> m_buffer;
+    std::array<char, buffer_size> m_buffer{};
 };
 
 /**
@@ -92,11 +94,14 @@ public:
     uint64_t stop() noexcept
     {
         m_stop = std::time(nullptr);
-        return m_stop - m_start;
+        return static_cast<uint64_t>(m_stop - m_start);
     }
 
     /// Return elapsed time
-    uint64_t elapsed() const noexcept { return m_stop - m_start; }
+    uint64_t elapsed() const noexcept
+    {
+        return static_cast<uint64_t>(m_stop - m_start);
+    }
 
     /**
      * Calculate ratio: value divided by elapsed time.
@@ -120,6 +125,10 @@ private:
 
 std::string human_readable_duration(uint64_t seconds);
 
+std::string human_readable_duration(std::chrono::milliseconds ms);
+
+std::string get_password();
+
 } // namespace util
 
 #endif // OSM2PGSQL_UTIL_HPP
