@@ -1,7 +1,7 @@
 
-local dtable = osm2pgsql.define_node_table('osm2pgsql_test_point', {
+local dtable = osm2pgsql.define_way_table('osm2pgsql_test_line', {
     { column = 'tags', type = 'hstore' },
-    { column = 'geom', type = 'point' },
+    { column = 'geom', type = 'linestring' },
 }, { schema = 'myschema' })
 
 local delete_keys = {
@@ -12,13 +12,14 @@ local delete_keys = {
 
 local clean_tags = osm2pgsql.make_clean_tags_func(delete_keys)
 
-function osm2pgsql.process_node(object)
+function osm2pgsql.process_way(object)
     if clean_tags(object.tags) then
         return
     end
 
     dtable:add_row({
-        tags = object.tags
+        tags = object.tags,
+        geom = { create = 'line' }
     })
 end
 
