@@ -18,6 +18,7 @@
 
 import logging
 from os import path as op
+from io import StringIO
 import os
 import subprocess
 import sys
@@ -50,14 +51,18 @@ def setUpModule():
     logging.info("Setting up test database")
     dbname = CONFIG['test_database']
 
-    with psycopg2.connect("dbname='template1'") as conn:
+    conn = psycopg2.connect("dbname='template1'")
+    try:
         conn.autocommit = True
         with conn.cursor() as cur:
             cur.execute('DROP DATABASE IF EXISTS "{}"'.format(dbname))
             cur.execute("CREATE DATABASE \"{}\" WITH ENCODING 'UTF8'"
                         .format(dbname))
+    finally:
+        conn.close()
 
-    with psycopg2.connect("dbname='{}'".format(dbname)) as conn:
+    conn = psycopg2.connect("dbname='{}'".format(dbname))
+    try:
         conn.autocommit = True
         with conn.cursor() as cur:
             # Check if there is a dataspace, we will skip tests otherwise.
@@ -68,17 +73,22 @@ def setUpModule():
 
             cur.execute('CREATE EXTENSION postgis')
             cur.execute('CREATE EXTENSION hstore')
+    finally:
+        conn.close()
 
 def tearDownModule():
     """ Destroy the global database.
     """
     logging.info("Cleaning up test database")
 
-    with psycopg2.connect("dbname='template1'") as conn:
+    conn = psycopg2.connect("dbname='template1'")
+    try:
+        conn.autocommit = True
         with conn.cursor() as cur:
-            conn.autocommit = True
             cur.execute('DROP DATABASE IF EXISTS "{}"'
                         .format(CONFIG['test_database']))
+    finally:
+        conn.close()
 
 ########################################################################
 #
@@ -111,13 +121,17 @@ class BaseRunner(object):
             cls.skipTest(None, "No Lua configured.")
         if 'tablespacetest' in cls.extra_params and not CONFIG['have_lua']:
             cls.skipTest(None, "Test tablespace 'tablespacetest' not configured.")
-        with psycopg2.connect("dbname='{}'".format(CONFIG['test_database'])) as conn:
+        conn = psycopg2.connect("dbname='{}'".format(CONFIG['test_database']))
+        try:
             with conn.cursor() as cur:
                 for t in ('nodes', 'ways', 'rels', 'point', 'line', 'roads', 'polygon'):
                     cur.execute("DROP TABLE IF EXISTS planet_osm_" + t)
                 cur.execute("DROP SCHEMA IF EXISTS osm CASCADE")
                 if cls.schema:
                     cur.execute("CREATE SCHEMA " + cls.schema)
+            conn.commit()
+        finally:
+            conn.close()
 
         if cls.import_file:
             cls.run_import(cls.get_def_params() + cls.extra_params,
@@ -134,8 +148,11 @@ class BaseRunner(object):
             BaseRunner.conn.close()
             BaseRunner.conn = None
         if cls.schema:
-            with psycopg2.connect("dbname='{}'".format(CONFIG['test_database'])) as conn:
+            conn = psycopg2.connect("dbname='{}'".format(CONFIG['test_database']))
+            try:
                 conn.cursor().execute("DROP SCHEMA IF EXISTS {} CASCADE".format(cls.schema))
+            finally:
+                conn.close()
         try:
             os.remove('flat.nodes')
         except (FileNotFoundError):
@@ -177,6 +194,27 @@ class BaseRunner(object):
             raise RuntimeError("Import failed.")
 
 
+    @classmethod
+    def import_from_string(cls, content, format='opl'):
+        cmdline = [CONFIG['executable']]
+        params = cls.get_def_params() + cls.extra_params
+        if not '-d' in params and not '--database' in params:
+            cmdline.extend(('-d', CONFIG['test_database']))
+        cmdline.extend(params)
+        cmdline.extend(('-r', format, '-'))
+        logging.info("Executing command: {}".format(' '.join(cmdline)))
+
+        proc = subprocess.Popen(cmdline, stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+        (outp, err) = proc.communicate(input=content.encode('utf-8'))
+
+        if proc.returncode == 0:
+            logging.debug(err.decode('utf-8'))
+        else:
+            logging.warning(err.decode('utf-8'))
+            raise RuntimeError("Import failed.")
+
     def assert_count(self, count, table, where=None):
         if self.schema:
             table = self.schema + '.' + table
@@ -287,6 +325,12 @@ class PgsqlMercGeomTests(object):
                         """SELECT round(sum(cast(ST_Area(way) as numeric)),0)
                            FROM planet_osm_polygon""")
 
+    def test_basic_polygon_way_area(self):
+        area = 1272140891 if self.use_lua_tagtransform else 1247245413
+        self.assert_sql(area,
+                        """SELECT round(sum(cast(way_area as numeric)), 0)
+                           FROM planet_osm_polygon""")
+
     def test_basic_line_length(self):
         self.assert_sql(4211350,
                         'SELECT round(sum(ST_Length(way))) FROM planet_osm_line')
@@ -404,6 +448,13 @@ class MultipolygonTests(object):
             sql += " AND name = '{}'".format(name)
         self.assert_sql(area, sql.format(osm_id, k, v, name))
 
+    def assert_sum_way_area(self, area, osm_id, k, v, name=None):
+        sql = """SELECT round(sum(way_area::numeric), 0) FROM planet_osm_polygon
+                 WHERE osm_id = {} AND "{}" = '{}'"""
+        if name is not None:
+            sql += " AND name = '{}'".format(name)
+        self.assert_sql(area, sql.format(osm_id, k, v, name))
+
     def assert_num_rows(self, area, osm_id, k, v, name = None):
         sql = """SELECT count(*) FROM planet_osm_polygon
                  WHERE osm_id = {} AND "{}" = '{}'"""
@@ -448,6 +499,9 @@ class MultipolygonTests(object):
     def test_multipolygon_two_outer(self):
         self.assert_sum_area(17581, -13, 'landuse', 'farmland', 'Name_rel9')
 
+    def test_multipolygon_two_outer_way_area(self):
+        self.assert_sum_way_area(17581, -13, 'landuse', 'farmland', 'Name_rel9')
+
     def test_multipolygon_two_outer_num(self):
         if '-G' in self.extra_params:
             self.assert_num_geoms(2, -13, 'landuse', 'farmland', 'Name_rel9')
@@ -457,6 +511,9 @@ class MultipolygonTests(object):
     def test_nested_outer_ways_area(self):
         self.assert_sum_area(16169, -7, 'landuse', 'farmland', 'Name_rel15')
 
+    def test_nested_outer_ways_way_area(self):
+        self.assert_sum_way_area(16169, -7, 'landuse', 'farmland', 'Name_rel15')
+
     def test_nested_outer_ways_num(self):
         if '-G' in self.extra_params:
             self.assert_num_geoms(2, -7, 'landuse', 'farmland', 'Name_rel15')
@@ -489,10 +546,14 @@ class MultipolygonTests(object):
     def test_tags_on_outer_absence_relation(self):
         self.assert_num_rows(0, -33, 'natural', 'water')
 
-    def test_tags_on_relation_includes_relatoin(self):
+    def test_tags_on_relation_includes_relation(self):
         self.assert_sum_area(29155 if self.update else 68494,
                              -29, 'natural', 'water')
 
+    def test_tags_on_relation_includes_relation_way_area(self):
+        self.assert_sum_way_area(29155 if self.update else 68494,
+                                 -29, 'natural', 'water')
+
     def test_tags_relation_outer_1_absence(self):
         self.assert_count(0, 'planet_osm_polygon', where='osm_id = 109')
 
@@ -511,12 +572,18 @@ class MultipolygonTests(object):
     def test_tags_on_relation_and_way_relation_presence(self):
         self.assert_sum_area(10377, -39, 'landuse', 'forest')
 
+    def test_tags_on_relation_and_way_relation_presence_way_area(self):
+        self.assert_sum_way_area(10378, -39, 'landuse', 'forest')
+
     def test_tags_on_relation_and_way_way_presence(self):
         self.assert_count(1, 'planet_osm_polygon', where='osm_id = 138')
 
     def test_different_tags_on_relation_and_way_relation_presence(self):
         self.assert_sum_area(12397, -40, 'landuse', 'forest')
 
+    def test_different_tags_on_relation_and_way_relation_presence_way_area(self):
+        self.assert_sum_way_area(12397, -40, 'landuse', 'forest')
+
     def test_different_tags_on_relation_and_way_way_presence(self):
         self.assert_count(1, 'planet_osm_polygon', where='osm_id = 140')
 
@@ -828,3 +895,21 @@ class TestDBOutputSchema(BaseUpdateRunnerWithOutputSchema, unittest.TestCase,
                          PgsqlBaseTests):
     extra_params = ['--slim', '--output-pgsql-schema=osm']
 
+# Bad data tests
+
+class TestBadOSMData(BaseRunner, unittest.TestCase):
+
+    extra_params = ['--slim']
+
+    def test_large_relation(self):
+        content = StringIO()
+        content.write('n1 x45 y34\n')
+        content.write('r1 Ttype=multipolygon M')
+        for _ in range(33000):
+            content.write('n1@,')
+        content.write('n1@')
+        self.import_from_string(content.getvalue())
+
+        self.assert_count(1, 'planet_osm_nodes')
+        self.assert_count(0, 'planet_osm_rels')
+
