@@ -3,19 +3,19 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
 #include <catch.hpp>
 
-#include "format.hpp"
-
 #include "domain-matcher.hpp"
 
 #include <osmium/builder/osm_object_builder.hpp>
 #include <osmium/memory/buffer.hpp>
-#include <osmium/osm.hpp>
+#include <osmium/osm/tag.hpp>
+
+#include <cstring>
 
 static osmium::Tag &fill_buffer(osmium::memory::Buffer &buffer, char const *key,
                                 char const *value)
