@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -83,7 +83,8 @@ struct tile_output_set
     void expire_centroids(expire_tiles &et)
     {
         for (auto const &t : tiles) {
-            double x0, y0;
+            double x0 = NAN;
+            double y0 = NAN;
             t.to_centroid(x0, y0);
             et.from_bbox(x0, y0, x0, y0);
         }
