@@ -3,14 +3,13 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
 #include <catch.hpp>
 
 #include <algorithm>
-#include <cassert>
 
 #include <osmium/osm/crc.hpp>
 #include <osmium/osm/crc_zlib.hpp>
@@ -44,6 +43,26 @@ struct options_slim_default
     }
 };
 
+struct options_slim_with_lc_prefix
+{
+    static options_t options(testing::pg::tempdb_t const &tmpdb)
+    {
+        options_t o = testing::opt_t().slim(tmpdb);
+        o.prefix = "pre";
+        return o;
+    }
+};
+
+struct options_slim_with_uc_prefix
+{
+    static options_t options(testing::pg::tempdb_t const &tmpdb)
+    {
+        options_t o = testing::opt_t().slim(tmpdb);
+        o.prefix = "PRE";
+        return o;
+    }
+};
+
 struct options_slim_with_schema
 {
     static options_t options(testing::pg::tempdb_t const &tmpdb)
@@ -54,16 +73,6 @@ struct options_slim_with_schema
     }
 };
 
-struct options_slim_dense_cache
-{
-    static options_t options(testing::pg::tempdb_t const &tmpdb)
-    {
-        options_t o = options_slim_default::options(tmpdb);
-        o.alloc_chunkwise = ALLOC_DENSE;
-        return o;
-    }
-};
-
 struct options_flat_node_cache
 {
     static options_t options(testing::pg::tempdb_t const &tmpdb)
@@ -76,44 +85,13 @@ struct options_ram_optimized
 {
     static options_t options(testing::pg::tempdb_t const &)
     {
-        options_t o = testing::opt_t();
-        o.alloc_chunkwise = ALLOC_SPARSE | ALLOC_DENSE;
-        return o;
+        return testing::opt_t();
     }
 };
 
-struct options_ram_flatnode
-{
-    static options_t options(testing::pg::tempdb_t const &)
-    {
-        options_t o = testing::opt_t().flatnodes();
-        o.alloc_chunkwise = ALLOC_SPARSE | ALLOC_DENSE;
-        return o;
-    }
-};
-
-TEST_CASE("elem_cache_t")
-{
-    elem_cache_t<int, 10> cache;
-
-    cache.set(3, new int{23});
-    cache.set(5, new int{42});
-    REQUIRE(*cache.get(3) == 23);
-    REQUIRE(*cache.get(5) == 42);
-    REQUIRE(cache.get(2) == nullptr);
-    cache.set(2, new int{56});
-    REQUIRE(*cache.get(2) == 56);
-    cache.set(3, new int{0});
-    REQUIRE(*cache.get(3) == 0);
-    cache.clear();
-    REQUIRE(cache.get(1) == nullptr);
-    REQUIRE(cache.get(2) == nullptr);
-    REQUIRE(cache.get(3) == nullptr);
-}
-
 TEMPLATE_TEST_CASE("middle import", "", options_slim_default,
-                   options_slim_with_schema, options_slim_dense_cache,
-                   options_ram_optimized, options_ram_flatnode)
+                   options_slim_with_lc_prefix, options_slim_with_uc_prefix,
+                   options_slim_with_schema, options_ram_optimized)
 {
     options_t const options = TestType::options(db);
     testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
@@ -131,12 +109,15 @@ TEMPLATE_TEST_CASE("middle import", "", options_slim_default,
         conn.exec("CREATE SCHEMA IF NOT EXISTS osm;");
     }
 
-    auto mid = options.slim
-                   ? std::shared_ptr<middle_t>(new middle_pgsql_t{&options})
-                   : std::shared_ptr<middle_t>(new middle_ram_t{&options});
-
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+    auto mid = create_middle(thread_pool, options);
     mid->start();
 
+    output_requirements requirements;
+    requirements.full_ways = true;
+    requirements.full_relations = true;
+    mid->set_requirements(requirements);
+
     auto const mid_q = mid->get_query_instance();
 
     test_buffer_t buffer;
@@ -239,10 +220,9 @@ TEMPLATE_TEST_CASE("middle import", "", options_slim_default,
         crc.update(rel);
         CHECK(orig_crc().checksum() == crc().checksum());
 
-        // retrive the supporting ways
-        rolelist_t roles;
-        REQUIRE(mid_q->rel_way_members_get(rel, &roles, &outbuf) == 3);
-        REQUIRE(roles.size() == 3);
+        // retrieve the supporting ways
+        REQUIRE(mid_q->rel_members_get(rel, &outbuf,
+                                       osmium::osm_entity_bits::way) == 3);
 
         for (auto &w : outbuf.select<osmium::Way>()) {
             REQUIRE(w.id() >= 10);
@@ -294,9 +274,10 @@ static bool no_node(std::shared_ptr<middle_pgsql_t> const &mid, osmid_t id)
 }
 
 TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
-                   options_slim_default, options_slim_dense_cache,
-                   options_flat_node_cache)
+                   options_slim_default, options_flat_node_cache)
 {
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+
     options_t options = TestType::options(db);
 
     testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
@@ -318,7 +299,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
     // and add some nodes. Does this in its own scope so that the mid is
     // closed properly.
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
 
         mid->start();
 
@@ -336,7 +317,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
 
     SECTION("Added nodes are there and no others")
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         check_node(mid, node10);
@@ -348,7 +329,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
     SECTION("Delete existing and non-existing node")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->node(node5d);
@@ -364,7 +345,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_node(mid, 5));
@@ -377,7 +358,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
     SECTION("Change (delete and set) existing and non-existing node")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->node(node10d);
@@ -393,7 +374,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             check_node(mid, node10a);
@@ -405,7 +386,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
     SECTION("Add new node")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->node(node12);
@@ -420,7 +401,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_node(mid, 5));
@@ -487,9 +468,10 @@ static bool no_way(std::shared_ptr<middle_pgsql_t> const &mid, osmid_t id)
 }
 
 TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
-                   options_slim_default, options_slim_dense_cache,
-                   options_flat_node_cache)
+                   options_slim_default, options_flat_node_cache)
 {
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+
     options_t options = TestType::options(db);
 
     testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
@@ -515,7 +497,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
     // add some ways. Does this in its own scope so that the mid is closed
     // properly.
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         mid->way(way20);
@@ -532,7 +514,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
 
     SECTION("Added ways are there and no others")
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         REQUIRE(no_way(mid, 5));
@@ -544,7 +526,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
     SECTION("Delete existing and non-existing way")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->way(way5d);
@@ -560,7 +542,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_way(mid, 5));
@@ -573,7 +555,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
     SECTION("Change (delete and set) existing and non-existing way")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->way(way20d);
@@ -591,7 +573,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_way(mid, 5));
@@ -605,7 +587,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
     SECTION("Add new way")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->way(way22);
@@ -620,7 +602,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_way(mid, 5));
@@ -633,8 +615,10 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
 }
 
 TEMPLATE_TEST_CASE("middle: add way with attributes", "", options_slim_default,
-                   options_slim_dense_cache, options_flat_node_cache)
+                   options_flat_node_cache)
 {
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+
     options_t options = TestType::options(db);
 
     SECTION("With attributes") { options.extra_attributes = true; }
@@ -663,7 +647,7 @@ TEMPLATE_TEST_CASE("middle: add way with attributes", "", options_slim_default,
         "osm_version=123,osm_timestamp=2009-02-13T23:31:30Z,osm_changeset=456");
 
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         mid->way(way20);
@@ -678,7 +662,7 @@ TEMPLATE_TEST_CASE("middle: add way with attributes", "", options_slim_default,
     options.append = true;
 
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         check_way(mid,
@@ -717,9 +701,10 @@ static bool no_relation(std::shared_ptr<middle_pgsql_t> const &mid, osmid_t id)
 }
 
 TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
-                   options_slim_default, options_slim_dense_cache,
-                   options_flat_node_cache)
+                   options_slim_default, options_flat_node_cache)
 {
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+
     options_t options = TestType::options(db);
 
     testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
@@ -745,7 +730,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
     // add some relations. Does this in its own scope so that the mid is closed
     // properly.
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         mid->relation(relation30);
@@ -761,7 +746,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
 
     SECTION("Added relations are there and no others")
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         REQUIRE(no_relation(mid, 5));
@@ -773,7 +758,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
     SECTION("Delete existing and non-existing relation")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->relation(relation5d);
@@ -788,7 +773,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_relation(mid, 5));
@@ -801,7 +786,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
     SECTION("Change (delete and set) existing and non-existing relation")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->relation(relation30d);
@@ -818,7 +803,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_relation(mid, 5));
@@ -832,7 +817,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
     SECTION("Add new relation")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->relation(relation32);
@@ -846,7 +831,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
         }
         {
             // Check with a new mid.
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             REQUIRE(no_relation(mid, 5));
@@ -859,9 +844,10 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
 }
 
 TEMPLATE_TEST_CASE("middle: add relation with attributes", "",
-                   options_slim_default, options_slim_dense_cache,
-                   options_flat_node_cache)
+                   options_slim_default, options_flat_node_cache)
 {
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+
     options_t options = TestType::options(db);
 
     SECTION("With attributes") { options.extra_attributes = true; }
@@ -887,7 +873,7 @@ TEMPLATE_TEST_CASE("middle: add relation with attributes", "",
         "osm_version=123,osm_timestamp=2009-02-13T23:31:30Z,osm_changeset=456");
 
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         mid->relation(relation30);
@@ -901,7 +887,7 @@ TEMPLATE_TEST_CASE("middle: add relation with attributes", "",
     options.append = true;
 
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         check_relation(mid, options.extra_attributes ? relation30_attr_tags
@@ -910,8 +896,10 @@ TEMPLATE_TEST_CASE("middle: add relation with attributes", "",
 }
 
 TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
-                   options_slim_dense_cache, options_flat_node_cache)
+                   options_flat_node_cache)
 {
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+
     options_t options = TestType::options(db);
 
     testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
@@ -936,7 +924,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
     // add some nodes and ways. Does this in its own scope so that the mid is
     // closed properly.
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         full_dependency_manager_t dependency_manager{mid};
         mid->start();
 
@@ -965,7 +953,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
 
     SECTION("Single way affected")
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         full_dependency_manager_t dependency_manager{mid};
         mid->start();
 
@@ -985,7 +973,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
     SECTION("Two ways affected")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->way(way22);
@@ -994,7 +982,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
             check_way(mid, way22);
         }
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             full_dependency_manager_t dependency_manager{mid};
             mid->start();
 
@@ -1017,7 +1005,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
     SECTION("Change way so the changing node isn't in it any more")
     {
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             mid->start();
 
             mid->way(way20d);
@@ -1030,7 +1018,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
         }
 
         {
-            auto mid = std::make_shared<middle_pgsql_t>(&options);
+            auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
             full_dependency_manager_t dependency_manager{mid};
             mid->start();
 
@@ -1045,8 +1033,10 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
 }
 
 TEMPLATE_TEST_CASE("middle: change nodes in relation", "", options_slim_default,
-                   options_slim_dense_cache, options_flat_node_cache)
+                   options_flat_node_cache)
 {
+    auto thread_pool = std::make_shared<thread_pool_t>(1U);
+
     options_t options = TestType::options(db);
 
     testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
@@ -1071,7 +1061,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in relation", "", options_slim_default,
     // add some nodes and ways. Does this in its own scope so that the mid is
     // closed properly.
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         mid->start();
 
         mid->node(node10);
@@ -1090,7 +1080,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in relation", "", options_slim_default,
 
     SECTION("Single relation directly affected")
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         full_dependency_manager_t dependency_manager{mid};
         mid->start();
 
@@ -1109,7 +1099,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in relation", "", options_slim_default,
 
     SECTION("Single relation indirectly affected (through way)")
     {
-        auto mid = std::make_shared<middle_pgsql_t>(&options);
+        auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
         full_dependency_manager_t dependency_manager{mid};
         mid->start();
 
