@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -15,7 +15,7 @@
 #include "taginfo-impl.hpp"
 #include "tagtransform.hpp"
 
-char const *TEST_PBF = "foo.pbf";
+char const *const TEST_PBF = "foo.pbf";
 
 static void bad_opt(std::vector<char const *> opts, char const *msg)
 {
