@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -13,6 +13,7 @@
 
 #include "common-import.hpp"
 #include "config.h"
+#include "reprojection.hpp"
 
 static testing::db::import_t db;
 
@@ -83,6 +84,6 @@ TEST_CASE("Projection setup")
 
     auto conn = db.connect();
 
-    CHECK(conn.require_scalar<std::string>(
+    CHECK(conn.result_as_string(
               "SELECT Find_SRID('public', 'planet_osm_roads', 'way')") == srid);
 }
