@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -29,14 +29,14 @@ TEST_CASE("config with schema should work")
     REQUIRE(1 == conn.get_count("pg_namespace", "nspname = 'myschema'"));
     REQUIRE(1 == conn.get_count("pg_tables", "schemaname = 'myschema'"));
 
-    REQUIRE(1362 == conn.get_count("myschema.osm2pgsql_test_point"));
+    REQUIRE(7103 == conn.get_count("myschema.osm2pgsql_test_line"));
 
     REQUIRE(1 ==
             conn.get_count("pg_proc",
-                           "proname = 'osm2pgsql_test_point_osm2pgsql_valid'"));
+                           "proname = 'osm2pgsql_test_line_osm2pgsql_valid'"));
 
     REQUIRE(1 == conn.get_count("pg_trigger"));
     REQUIRE(1 ==
             conn.get_count("pg_trigger",
-                           "tgname = 'osm2pgsql_test_point_osm2pgsql_valid'"));
+                           "tgname = 'osm2pgsql_test_line_osm2pgsql_valid'"));
 }
