@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -28,10 +28,10 @@ TEST_CASE("compute Z order")
         auto const sql = "SELECT highway FROM osm2pgsql_test_line"
                          " WHERE layer IS NULL ORDER BY z_order DESC"
                          " LIMIT 1 OFFSET {}"_format(i);
-        REQUIRE(expected[i] == conn.require_scalar<std::string>(sql));
+        REQUIRE(expected[i] == conn.result_as_string(sql));
     }
 
-    REQUIRE("residential" == conn.require_scalar<std::string>(
-                                 "SELECT highway FROM osm2pgsql_test_line "
-                                 "ORDER BY z_order DESC LIMIT 1 OFFSET 0"));
+    REQUIRE("residential" ==
+            conn.result_as_string("SELECT highway FROM osm2pgsql_test_line "
+                                  "ORDER BY z_order DESC LIMIT 1 OFFSET 0"));
 }
