@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -12,11 +12,11 @@
 #include "common-import.hpp"
 #include "pgsql.hpp"
 
-static testing::db::import_t db;
+static testing::db::import_t const db;
 
 TEST_CASE("Tablespace clause with no tablespace")
 {
-    REQUIRE(tablespace_clause("") == "");
+    REQUIRE(tablespace_clause("").empty());
 }
 
 TEST_CASE("Tablespace clause with tablespace")
